# Copyright 2012 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.


from django.core.urlresolvers import reverse
from client import Untappd
from models import UntappdProfile
from forms import UnlinkUntappdForm
from socialregistration.views import OAuthRedirect, OAuthCallback, SetupCallback

class UntappdRedirect(OAuthRedirect):
    client = Untappd
    template_name = 'connections/untappd/untappd.html'

class UntappdCallback(OAuthCallback):
    client = Untappd
    template_name = 'connections/untappd/untappd.html'
    
    def get_redirect(self):
        return reverse('untappd:setup')

class UntappdSetup(SetupCallback):
    client = Untappd
    profile = UntappdProfile
    template_name = 'connections/untappd/untappd.html'
    
    def get_lookup_kwargs(self, request, client):
        return {'untappd': client.get_user_info()['user_name']}



# -*- coding: latin-1 -*-
# Copyright 2010 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

import datetime
import os
import random

from django.conf import settings
from django.core.urlresolvers import reverse
from django.db import models
from django.db.models.signals import post_save
from django.db.models.signals import pre_save
from django.contrib.sites.models import Site
from django.contrib.auth.models import User
from django.utils import timezone

from pykeg import EPOCH

from pykeg.core import kb_common
from pykeg.core import fields
from pykeg.core import imagespecs
from pykeg.core import jsonfield
from pykeg.core import managers
from pykeg.core import stats
from pykeg.core.util import make_serial

from kegbot.util import units
from kegbot.util import util

from kegbot.api import models_pb2
from kegbot.api import protoutil

"""Django models definition for the kegbot database."""

class KegbotSite(models.Model):
  name = models.CharField(max_length=64, unique=True, default='default',
      editable=False)
  is_active = models.BooleanField(default=True,
      help_text='On/off switch for this site.')
  is_setup = models.BooleanField(default=True,
      help_text='True if the site has completed setup.',
      editable=False)
  epoch = models.PositiveIntegerField(default=EPOCH,
      help_text='Database epoch number.',
      editable=False)
  serial_number = models.TextField(max_length=128, editable=False,
      blank=True, default='',
      help_text='A unique id for this system.')

  def __str__(self):
    return self.name

  def full_url(self):
    return 'http://%s' % Site.objects.get_current().domain

  def GetStatsRecord(self):
    try:
      return SystemStats.objects.get(site=self)
    except SystemStats.DoesNotExist:
      return None

  def GetStats(self):
    record = self.GetStatsRecord()
    if record:
      return record.stats
    return {}

def _kegbotsite_pre_save(sender, instance, **kwargs):
  if not instance.serial_number:
    instance.serial_number = make_serial()
pre_save.connect(_kegbotsite_pre_save, sender=KegbotSite)

def _kegbotsite_post_save(sender, instance, **kwargs):
  """Creates a SiteSettings object if none already exists."""
  settings, _ = SiteSettings.objects.get_or_create(site=instance)
post_save.connect(_kegbotsite_post_save, sender=KegbotSite)

class SiteSettings(models.Model):
  VOLUME_DISPLAY_UNITS_CHOICES = (
    ('metric', 'Metric (mL, L)'),
    ('imperial', 'Imperial (oz, pint)'),
  )
  TEMPERATURE_DISPLAY_UNITS_CHOICES = (
    ('f', 'Fahrenheit'),
    ('c', 'Celsius'),
  )
  PRIVACY_CHOICES = (
    ('public', 'Public: Browsing does not require login'),
    ('members', 'Members only: Must log in to browse'),
    ('staff', 'Staff only: Only logged-in staff accounts may browse'),
  )
  DEFAULT_PRIVACY = 'public'

  site = models.OneToOneField(KegbotSite, related_name='settings')
  volume_display_units = models.CharField(max_length=64,
      choices=VOLUME_DISPLAY_UNITS_CHOICES, default='imperial',
      help_text='Unit system to use when displaying volumetric data.')
  temperature_display_units = models.CharField(max_length=64,
      choices=TEMPERATURE_DISPLAY_UNITS_CHOICES, default='f',
      help_text='Unit system to use when displaying temperature data.')
  title = models.CharField(max_length=64, blank=True, null=True,
      help_text='The title of this site. Example: "Kegbot San Francisco"')
  description = models.TextField(blank=True, null=True,
      help_text='Description of this site')
  background_image = models.ForeignKey('Picture', blank=True, null=True,
      help_text='Background for this site.')
  event_web_hook = models.URLField(blank=True, null=True,
      help_text='Web hook URL for newly-generated events.')
  google_analytics_id = models.CharField(blank=True, null=True, max_length=64,
      help_text='Set to your Google Analytics ID to enable tracking. '
      'Example: UA-XXXX-y')
  session_timeout_minutes = models.PositiveIntegerField(
      default=kb_common.DRINK_SESSION_TIME_MINUTES,
      help_text='Maximum time, in minutes, that a session may be idle (no pours) '
          'before it is considered to be finished.  '
          'Recommended value is %s.' % kb_common.DRINK_SESSION_TIME_MINUTES)
  privacy = models.CharField(max_length=63, choices=PRIVACY_CHOICES,
      default=DEFAULT_PRIVACY,
      help_text='Whole-system setting for system privacy.')
  guest_name = models.CharField(max_length=63, default='guest',
      help_text='Name to be shown in various places for unauthenticated pours.')
  guest_image = models.ForeignKey('Picture', blank=True, null=True,
      related_name='guest_images',
      help_text='Profile picture to be shown for unauthenticated pours.')
  default_user = models.ForeignKey(User, blank=True, null=True,
      help_text='Default user to set as owner for unauthenticated drinks. '
          'When set, the "guest" user will not be used. This is mostly '
          'useful for closed / single user systems.')
  registration_allowed = models.BooleanField(default=True,
      help_text='Whether to allow new user registration.')
  registration_confirmation = models.BooleanField(default=False,
      help_text='Whether registration requires e-mail confirmation.')

  allowed_hosts = models.TextField(blank=True, null=True, default='',
      help_text='List of allowed hostnames. If blank, validation is disabled.')

  class Meta:
    verbose_name_plural = "site settings"

  def GetSessionTimeoutDelta(self):
    return datetime.timedelta(minutes=self.session_timeout_minutes)

  def base_url(self):
    return 'http://%s' % (Site.objects.get_current(),)

  def reverse_full(self, *args, **kwargs):
    """Calls reverse, and returns a full URL (includes base_url())."""
    return '%s%s' % (self.base_url(), reverse(*args, **kwargs))

  @classmethod
  def get(cls):
    """Gets the default site settings."""
    site = KegbotSite.objects.get(name='default')
    return site.settings


class UserProfile(models.Model):
  """Extra per-User information."""
  def __str__(self):
    return "profile for %s" % (self.user,)

  def FacebookProfile(self):
    if 'socialregistration' not in settings.INSTALLED_APPS:
      return None
    qs = self.user.facebookprofile_set.all()
    if qs:
      return qs[0]

  def TwitterProfile(self):
    if 'socialregistration' not in settings.INSTALLED_APPS:
      return None
    qs = self.user.twitterprofile_set.all()
    if qs:
      return qs[0]

  def GetStatsRecord(self):
    try:
      return UserStats.objects.get(user=self)
    except UserStats.DoesNotExist:
      return None

  def GetStats(self):
    record = self.GetStatsRecord()
    if record:
      return record.stats
    return {}

  def RecomputeStats(self):
    self.user.stats.all().delete()
    last_d = self.user.drinks.valid().order_by('-time')
    if last_d:
      last_d[0]._UpdateUserStats()

  def GetApiKey(self):
    api_key, new = ApiKey.objects.get_or_create(user=self.user,
        defaults={'key': ApiKey.generate_key()})
    return api_key.key

  @models.permalink
  def get_absolute_url(self):
    return ('kb-drinker', (self.user.username,))

  user = models.OneToOneField(User)
  mugshot = models.ForeignKey('Picture', blank=True, null=True)

def _user_post_save(sender, instance, **kwargs):
  profile, new = UserProfile.objects.get_or_create(user=instance)
post_save.connect(_user_post_save, sender=User)


class ApiKey(models.Model):
  user = models.OneToOneField(User)
  key = models.CharField(max_length=127, editable=False, unique=True)
  active = models.BooleanField(default=True)

  def is_active(self):
    """Returns true if both the key and the key's user are active."""
    return self.active and self.user.is_active

  def regenerate(self):
    self.key = self.generate_key()
    self.save()

  @classmethod
  def generate_key(cls):
    '''Returns a new random key.'''
    return '%032x' % random.randint(0, 2**128 - 1)


class BeerDBModel(models.Model):
  class Meta:
    abstract = True
  added = models.DateTimeField(default=timezone.now, editable=False)
  edited = models.DateTimeField(editable=False)
  beerdb_id = models.CharField(blank=True, null=True, max_length=128)

  def save(self, *args, **kwargs):
    self.edited = timezone.now()
    super(BeerDBModel, self).save(*args, **kwargs)


class Brewer(BeerDBModel):
  """Describes a producer of beer."""
  PRODUCTION_CHOICES = (
    ('commercial', 'Commercial brewer'),
    ('homebrew', 'Home brewer'),
  )

  name = models.CharField(max_length=255,
      help_text='Name of the brewer')
  country = fields.CountryField(default='USA',
      help_text='Country of origin')
  origin_state = models.CharField(max_length=128,
      default='', blank=True, null=True,
      help_text='State of origin, if applicable')
  origin_city = models.CharField(max_length=128, default='', blank=True,
      null=True,
      help_text='City of origin, if known')
  production = models.CharField(max_length=128, choices=PRODUCTION_CHOICES,
      default='commercial')
  url = models.URLField(default='', blank=True, null=True,
      help_text='Brewer\'s home page')
  description = models.TextField(default='', blank=True, null=True,
      help_text='A short description of the brewer')
  image = models.ForeignKey('Picture', blank=True, null=True,
      related_name='beer_brewers')

  def __str__(self):
    return self.name


class BeerStyle(BeerDBModel):
  """Describes a named style of beer (Stout, IPA, etc)"""
  name = models.CharField(max_length=128,
      help_text='Name of the beer style')

  def __str__(self):
    return self.name


class BeerType(BeerDBModel):
  """Describes a specific kind of beer, by name, brewer, and style."""
  name = models.CharField(max_length=255)
  brewer = models.ForeignKey(Brewer)
  style = models.ForeignKey(BeerStyle)

  edition = models.CharField(max_length=255, blank=True, null=True,
      help_text='For seasonal or special edition beers, enter the '
          'year or other edition name')

  abv = models.FloatField(blank=True, null=True,
      help_text='Alcohol by volume, as percentage (0-100), if known')
  calories_oz = models.FloatField(blank=True, null=True,
      help_text='Calories per ounce of beer, if known')
  carbs_oz = models.FloatField(blank=True, null=True,
      help_text='Carbohydrates per ounce of beer, if known')

  original_gravity = models.FloatField(blank=True, null=True,
      help_text='Original gravity of the beer, if known')
  specific_gravity = models.FloatField(blank=True, null=True,
      help_text='Specific/final gravity of the beer, if known')

  image = models.ForeignKey('Picture', blank=True, null=True,
      related_name='beer_types')

  untappd_beer_id = models.IntegerField(blank=True, null=True,
      help_text='Untappd.com beer id for this beer, if known')

  def __str__(self):
    return "%s by %s" % (self.name, self.brewer)

  def GetImage(self):
    if self.image:
      return self.image
    if self.brewer.image:
      return self.brewer.image
    return None


class KegSize(models.Model):
  """ A convenient table of common Keg sizes """
  def __str__(self):
    gallons = units.Quantity(self.volume_ml).InUSGallons()
    return "%s [%.2f gal]" % (self.name, gallons)

  name = models.CharField(max_length=128)
  volume_ml = models.FloatField()


class KegTap(models.Model):
  """A physical tap of beer."""
  site = models.ForeignKey(KegbotSite, related_name='taps')
  name = models.CharField(max_length=128,
      help_text='The display name for this tap. Example: Main Tap.')
  meter_name = models.CharField(max_length=128,
      help_text='The name of the flow meter reporting to this tap. '
      'Example: kegboard.flow0')
  relay_name = models.CharField(max_length=128, blank=True, null=True,
      help_text='If a relay is attached to this tap, give its '
      'name here. Example: kegboard.relay0')
  ml_per_tick = models.FloatField(default=(1000.0/2200.0),
      help_text='mL per flow meter tick.  Common values: 0.166666666667 '
      '(SwissFlow), 0.454545454545 (Vision 2000)')
  description = models.TextField(blank=True, null=True)
  current_keg = models.OneToOneField('Keg', blank=True, null=True,
      related_name='current_tap')
  max_tick_delta = models.PositiveIntegerField(default=100)
  temperature_sensor = models.ForeignKey('ThermoSensor', blank=True, null=True)

  def __str__(self):
    return "%s: %s" % (self.meter_name, self.name)

  def Temperature(self):
    if self.temperature_sensor:
      last_rec = self.temperature_sensor.thermolog_set.all().order_by('-time')
      if last_rec:
        return last_rec[0]
    return None


class Keg(models.Model):
  """ Record for each installed Keg. """
  @models.permalink
  def get_absolute_url(self):
    return ('kb-keg', (str(self.id),))

  def full_volume(self):
    return self.size.volume_ml

  def served_volume(self):
    drinks = Drink.objects.filter(keg__exact=self, status__exact='valid')
    total = 0
    for d in drinks:
      total += d.volume_ml
    return total

  def spilled_volume(self):
    return self.spilled_ml

  def remaining_volume(self):
    return self.full_volume() - self.served_volume() - self.spilled_volume()

  def percent_full(self):
    result = float(self.remaining_volume()) / float(self.full_volume()) * 100
    result = max(min(result, 100), 0)
    return result

  def end_keg(self):
    if self.current_tap:
      tap = self.current_tap
      tap.current_keg = None
      tap.save()
    if self.status != 'offline':
      self.status = 'offline'
      self.save()

  def keg_age(self):
    if self.status == 'online':
      end = timezone.now()
    else:
      end = self.end_time
    return end - self.start_time

  def is_empty(self):
    return float(self.remaining_volume()) <= 0

  def is_active(self):
    return self.status == 'online'

  def previous(self):
    q = Keg.objects.filter(site=self.site, start_time__lt=self.start_time).order_by('-start_time')
    if q.count():
      return q[0]
    return None

  def next(self):
    q = Keg.objects.filter(site=self.site, start_time__gt=self.start_time).order_by('start_time')
    if q.count():
      return q[0]
    return None

  def GetStatsRecord(self):
    try:
      return KegStats.objects.get(keg=self)
    except KegStats.DoesNotExist:
      return None

  def GetStats(self):
    record = self.GetStatsRecord()
    if record:
      return record.stats
    return {}

  def RecomputeStats(self):
    self.stats.all().delete()
    last_d = self.drinks.valid().order_by('-start_time')
    if last_d:
      last_d[0]._UpdateKegStats()

  def Sessions(self):
    chunks = SessionChunk.objects.filter(keg=self).order_by('-start_time').select_related(depth=2)
    sessions = []
    sess = None
    for c in chunks:
      # Skip same sessions
      if c.session == sess:
        continue
      sess = c.session
      sessions.append(sess)
    return sessions

  def TopDrinkers(self):
    stats = self.GetStats()
    if not stats:
      return []
    ret = []
    entries = stats.get('volume_by_drinker', [])
    for entry in entries:
      username = str(entry.username)
      vol = entry.volume_ml
      try:
        user = User.objects.get(username=username)
      except User.DoesNotExist:
        continue  # should not happen
      ret.append((vol, user))
    ret.sort(reverse=True)
    return ret

  def __str__(self):
    return "Keg #%s - %s" % (self.id, self.type)

  site = models.ForeignKey(KegbotSite, related_name='kegs')
  type = models.ForeignKey('BeerType')
  size = models.ForeignKey(KegSize)
  start_time = models.DateTimeField(default=timezone.now)
  end_time = models.DateTimeField(default=timezone.now)
  status = models.CharField(max_length=128, choices=(
     ('online', 'online'),
     ('offline', 'offline'),
     ('coming soon', 'coming soon')))
  description = models.CharField(max_length=256, blank=True, null=True)
  origcost = models.FloatField(default=0, blank=True, null=True)
  spilled_ml = models.FloatField(default=0)
  notes = models.TextField(blank=True, null=True,
      help_text='Private notes about this keg, viewable only by admins.')

def _keg_pre_save(sender, instance, **kwargs):
  keg = instance
  # We don't need to do anything if the keg is still online.
  if keg.status != 'offline':
    return

  # Determine first drink date & set keg start date to it if earlier.
  drinks = keg.drinks.valid().order_by('time')
  if drinks:
    drink = drinks[0]
    if drink.time < keg.start_time:
      keg.start_time = drink.time

  # Determine last drink date & set keg end date to it if later.
  drinks = keg.drinks.valid().order_by('-time')
  if drinks:
    drink = drinks[0]
    if drink.time > keg.end_time:
      keg.end_time = drink.time

pre_save.connect(_keg_pre_save, sender=Keg)

def _keg_post_save(sender, instance, **kwargs):
  keg = instance
  SystemEvent.ProcessKeg(keg)

post_save.connect(_keg_post_save, sender=Keg)


class Drink(models.Model):
  """ Table of drinks records """
  class Meta:
    get_latest_by = 'time'
    ordering = ('-time',)

  @models.permalink
  def get_absolute_url(self):
    return ('kb-drink', (str(self.id),))

  def ShortUrl(self):
    return '%s%s' % (self.site.full_url(), reverse('kb-drink-short', args=(str(self.id),)))

  def Volume(self):
    return units.Quantity(self.volume_ml)

  def calories(self):
    if not self.keg or not self.keg.type:
      return 0
    ounces = self.Volume().InOunces()
    return self.keg.type.calories_oz * ounces

  def __str__(self):
    return "Drink %s:%i by %s" % (self.site.name, self.id, self.user)

  def _UpdateSystemStats(self):
    stats, created = SystemStats.objects.get_or_create(site=self.site)
    stats.Update(self)

  def _UpdateUserStats(self):
    if self.user:
      stats, created = UserStats.objects.get_or_create(user=self.user, site=self.site)
      stats.Update(self)

  def _UpdateKegStats(self):
    if self.keg:
      stats, created = KegStats.objects.get_or_create(keg=self.keg, site=self.site)
      stats.Update(self)

  def _UpdateSessionStats(self):
    if self.session:
      stats, created = SessionStats.objects.get_or_create(session=self.session, site=self.site)
      stats.Update(self)

  def PostProcess(self):
    self._UpdateSystemStats()
    self._UpdateUserStats()
    self._UpdateKegStats()
    self._UpdateSessionStats()
    SystemEvent.ProcessDrink(self)

  objects = managers.DrinkManager()

  site = models.ForeignKey(KegbotSite, related_name='drinks')

  # Ticks records the actual meter reading, which is never changed once
  # recorded.
  ticks = models.PositiveIntegerField(editable=False)

  # Volume is the actual volume of the drink.  Its initial value is a function
  # of `ticks`, but it may be adjusted, eg due to calibration or mis-recording.
  volume_ml = models.FloatField()

  time = models.DateTimeField()
  duration = models.PositiveIntegerField(blank=True, default=0)
  user = models.ForeignKey(User, null=True, blank=True, related_name='drinks')
  keg = models.ForeignKey(Keg, null=True, blank=True, related_name='drinks')
  status = models.CharField(max_length=128, choices = (
     ('valid', 'valid'),
     ('invalid', 'invalid'),
     ('deleted', 'deleted'),
     ), default = 'valid')
  session = models.ForeignKey('DrinkingSession',
      related_name='drinks', null=True, blank=True, editable=False)
  shout = models.TextField(blank=True, null=True,
      help_text='Comment from the drinker at the time of the pour.')
  tick_time_series = models.TextField(blank=True, null=True, editable=False,
      help_text='Tick update sequence that generated this drink')


class AuthenticationToken(models.Model):
  """A secret token to authenticate a user, optionally pin-protected."""
  class Meta:
    unique_together = ('auth_device', 'token_value')

  def __str__(self):
    auth_device = self.auth_device
    if auth_device == 'core.rfid':
      auth_device = 'RFID'
    elif auth_device == 'core.onewire':
      auth_device = 'OneWire'

    ret = "%s %s" % (auth_device, self.token_value)
    if self.nice_name:
      ret += " (%s)" % self.nice_name
    return ret

  site = models.ForeignKey(KegbotSite, related_name='tokens')
  auth_device = models.CharField(max_length=64)
  token_value = models.CharField(max_length=128)
  nice_name = models.CharField(max_length=256, blank=True, null=True,
      help_text='A human-readable alias for the token (eg "Guest Key").')
  pin = models.CharField(max_length=256, blank=True, null=True)
  user = models.ForeignKey(User, blank=True, null=True,
      related_name='tokens')
  enabled = models.BooleanField(default=True)
  created_time = models.DateTimeField(auto_now_add=True)
  expire_time = models.DateTimeField(blank=True, null=True)

  def get_auth_device(self):
    auth_device = self.auth_device
    if auth_device == 'core.rfid':
      auth_device = 'RFID'
    elif auth_device == 'core.onewire':
      auth_device = 'OneWire'
    return auth_device

  def IsAssigned(self):
    return self.user is not None

  def IsActive(self):
    if not self.enabled:
      return False
    if not self.expire_time:
      return True
    return timezone.now() < self.expire_time

def _auth_token_pre_save(sender, instance, **kwargs):
  if instance.auth_device in kb_common.AUTH_MODULE_NAMES_HEX_VALUES:
    instance.token_value = instance.token_value.lower()

pre_save.connect(_auth_token_pre_save, sender=AuthenticationToken)

class _AbstractChunk(models.Model):
  class Meta:
    abstract = True
    get_latest_by = 'start_time'
    ordering = ('-start_time',)

  start_time = models.DateTimeField()
  end_time = models.DateTimeField()
  volume_ml = models.FloatField(default=0)

  def Duration(self):
    return self.end_time - self.start_time

  def _AddDrinkNoSave(self, drink):
    session_delta = drink.site.settings.GetSessionTimeoutDelta()
    session_end = drink.time + session_delta

    if self.start_time > drink.time:
      self.start_time = drink.time
    if self.end_time < session_end:
      self.end_time = session_end
    self.volume_ml += drink.volume_ml

  def AddDrink(self, drink):
    self._AddDrinkNoSave(drink)
    self.save()


class DrinkingSession(_AbstractChunk):
  """A collection of contiguous drinks. """
  class Meta:
    get_latest_by = 'start_time'
    ordering = ('-start_time',)

  objects = managers.SessionManager()
  site = models.ForeignKey(KegbotSite, related_name='sessions')
  name = models.CharField(max_length=256, blank=True, null=True)

  def __str__(self):
    return "Session #%s: %s" % (self.id, self.start_time)

  def HighlightPicture(self):
    pictures = self.pictures.all().order_by('-time')
    if pictures:
      return pictures[0]
    chunks = self.user_chunks.filter(user__ne=None).order_by('-volume_ml')
    if chunks:
      mugshot = chunks[0].user.get_profile().mugshot
      return mugshot

  def OtherPictures(self):
    pictures = self.pictures.all().order_by('-time')
    if pictures:
      return pictures[1:]
    return []

  def RecomputeStats(self):
    self.stats.all().delete()
    try:
      last_d = self.drinks.valid().latest()
      last_d._UpdateSessionStats()
    except Drink.DoesNotExist:
      pass

  @models.permalink
  def get_absolute_url(self):
    dt = self.start_time
    if settings.USE_TZ:
      dt = timezone.localtime(dt)
    return ('kb-session-detail', (), {
      'year' : dt.year,
      'month' : dt.month,
      'day' : dt.day,
      'pk' : self.pk})

  def GetStatsRecord(self):
    try:
      return SessionStats.objects.get(session=self)
    except SessionStats.DoesNotExist:
      return None

  def GetStats(self):
    record = self.GetStatsRecord()
    if record:
      return record.stats
    return {}

  def summarize_drinkers(self):
    def fmt(user):
      url = '/drinkers/%s/' % (user.username,)
      return '<a href="%s">%s</a>' % (url, user.username)
    chunks = self.user_chunks.all().order_by('-volume_ml')
    users = tuple(c.user for c in chunks)
    names = tuple(fmt(u) for u in users if u)

    if None in users:
      guest_trailer = ' (and possibly others)'
    else:
      guest_trailer = ''

    num = len(names)
    if num == 0:
      return 'no known drinkers'
    elif num == 1:
      ret = names[0]
    elif num == 2:
      ret = '%s and %s' % names
    elif num == 3:
      ret = '%s, %s and %s' % names
    else:
      if guest_trailer:
        return '%s, %s and at least %i others' % (names[0], names[1], num-2)
      else:
        return '%s, %s and %i others' % (names[0], names[1], num-2)

    return '%s%s' % (ret, guest_trailer)

  def GetTitle(self):
    if self.name:
      return self.name
    else:
      if self.id:
        return 'Session %s' % (self.id,)
      else:
        # Not yet saved.
        return 'New Session'

  def AddDrink(self, drink):
    super(DrinkingSession, self).AddDrink(drink)
    session_delta = drink.site.settings.GetSessionTimeoutDelta()

    defaults = {
      'start_time': drink.time,
      'end_time': drink.time + session_delta,
    }

    # Update or create a SessionChunk.
    chunk, created = SessionChunk.objects.get_or_create(session=self,
        user=drink.user, keg=drink.keg, defaults=defaults)
    chunk.AddDrink(drink)

    # Update or create a UserSessionChunk.
    chunk, created = UserSessionChunk.objects.get_or_create(session=self,
        site=drink.site, user=drink.user, defaults=defaults)
    chunk.AddDrink(drink)

    # Update or create a KegSessionChunk.
    chunk, created = KegSessionChunk.objects.get_or_create(session=self,
        site=drink.site, keg=drink.keg, defaults=defaults)
    chunk.AddDrink(drink)

  def UserChunksByVolume(self):
    chunks = self.user_chunks.all().order_by('-volume_ml')
    return chunks

  def IsActiveNow(self):
    return self.IsActive(timezone.now())

  def IsActive(self, now):
    return self.end_time > now

  def Rebuild(self):
    self.volume_ml = 0
    self.chunks.all().delete()
    self.user_chunks.all().delete()
    self.keg_chunks.all().delete()

    drinks = self.drinks.valid()
    if not drinks:
      # TODO(mikey): cancel/delete the session entirely.  As it is, session will
      # remain a placeholder.
      return

    session_delta = self.site.settings.GetSessionTimeoutDelta()
    min_time = None
    max_time = None
    for d in drinks:
      self.AddDrink(d)
      if min_time is None or d.time < min_time:
        min_time = d.time
      if max_time is None or d.time > max_time:
        max_time = d.time
    self.start_time = min_time
    self.end_time = max_time + session_delta
    self.save()

  @classmethod
  def AssignSessionForDrink(cls, drink):
    # Return existing session if already assigned.
    if drink.session:
      return drink.session

    # Return last session if one already exists
    q = drink.site.sessions.all().order_by('-end_time')[:1]
    if q and q[0].IsActive(drink.time):
      session = q[0]
      session.AddDrink(drink)
      drink.session = session
      drink.save()
      return session

    # Create a new session
    session = cls(start_time=drink.time, end_time=drink.time,
        site=drink.site)
    session.save()
    session.AddDrink(drink)
    drink.session = session
    drink.save()
    return session


class SessionChunk(_AbstractChunk):
  """A specific user and keg contribution to a session."""
  class Meta:
    unique_together = ('session', 'user', 'keg')
    get_latest_by = 'start_time'
    ordering = ('-start_time',)

  session = models.ForeignKey(DrinkingSession, related_name='chunks')
  user = models.ForeignKey(User, related_name='session_chunks', blank=True,
      null=True)
  keg = models.ForeignKey(Keg, related_name='session_chunks', blank=True,
      null=True)


class UserSessionChunk(_AbstractChunk):
  """A specific user's contribution to a session (spans all kegs)."""
  class Meta:
    unique_together = ('session', 'user')
    get_latest_by = 'start_time'
    ordering = ('-start_time',)

  site = models.ForeignKey(KegbotSite, related_name='user_chunks')
  session = models.ForeignKey(DrinkingSession, related_name='user_chunks')
  user = models.ForeignKey(User, related_name='user_session_chunks', blank=True,
      null=True)

  def GetTitle(self):
    return self.session.GetTitle()

  def GetDrinks(self):
    return self.session.drinks.filter(user=self.user).order_by('time')


class KegSessionChunk(_AbstractChunk):
  """A specific keg's contribution to a session (spans all users)."""
  class Meta:
    unique_together = ('session', 'keg')
    get_latest_by = 'start_time'
    ordering = ('-start_time',)

  objects = managers.SessionManager()
  site = models.ForeignKey(KegbotSite, related_name='keg_chunks')
  session = models.ForeignKey(DrinkingSession, related_name='keg_chunks')
  keg = models.ForeignKey(Keg, related_name='keg_session_chunks', blank=True,
      null=True)

  def GetTitle(self):
    return self.session.GetTitle()


class ThermoSensor(models.Model):
  site = models.ForeignKey(KegbotSite, related_name='thermosensors')
  raw_name = models.CharField(max_length=256)
  nice_name = models.CharField(max_length=128)

  def __str__(self):
    if self.nice_name:
      return '%s (%s)' % (self.nice_name, self.raw_name)
    return self.raw_name

  def LastLog(self):
    try:
      return self.thermolog_set.latest()
    except Thermolog.DoesNotExist:
      return None


class Thermolog(models.Model):
  """ A log from an ITemperatureSensor device of periodic measurements. """
  class Meta:
    get_latest_by = 'time'
    ordering = ('-time',)

  site = models.ForeignKey(KegbotSite, related_name='thermologs')
  sensor = models.ForeignKey(ThermoSensor)
  temp = models.FloatField()
  time = models.DateTimeField()

  def __str__(self):
    return '%s %.2f C / %.2f F [%s]' % (self.sensor, self.TempC(),
        self.TempF(), self.time)

  def TempC(self):
    return self.temp

  def TempF(self):
    return util.CtoF(self.temp)


class _StatsModel(models.Model):
  STATS_BUILDER = None

  class Meta:
    abstract = True

  def Update(self, drink, force=False):
    previous = None
    try:
      if not force and self.stats:
        previous = protoutil.DictToProtoMessage(self.stats, models_pb2.Stats())
    except TypeError, e:
      pass
    builder = self.STATS_BUILDER(drink, previous)
    self.stats = protoutil.ProtoMessageToDict(builder.Build())
    self.save()

  site = models.ForeignKey(KegbotSite)
  time = models.DateTimeField(default=timezone.now)
  stats = jsonfield.JSONField()


class SystemStats(_StatsModel):
  STATS_BUILDER = stats.SystemStatsBuilder

  def __str__(self):
    return 'SystemStats for %s' % self.site


class UserStats(_StatsModel):
  class Meta:
    unique_together = ('site', 'user')
  STATS_BUILDER = stats.DrinkerStatsBuilder
  user = models.ForeignKey(User, blank=True, null=True, related_name='stats')

  def __str__(self):
    return 'UserStats for %s' % self.user


class KegStats(_StatsModel):
  STATS_BUILDER = stats.KegStatsBuilder
  keg = models.ForeignKey(Keg, unique=True, related_name='stats')
  completed = models.BooleanField(default=False)

  def __str__(self):
    return 'KegStats for %s' % self.keg


class SessionStats(_StatsModel):
  STATS_BUILDER = stats.SessionStatsBuilder
  session = models.ForeignKey(DrinkingSession, unique=True, related_name='stats')
  completed = models.BooleanField(default=False)

  def __str__(self):
    return 'SessionStats for %s' % self.session


class SystemEvent(models.Model):
  class Meta:
    ordering = ('-id',)
    get_latest_by = 'time'

  KINDS = (
      ('drink_poured', 'Drink poured'),
      ('session_started', 'Session started'),
      ('session_joined', 'User joined session'),
      ('keg_tapped', 'Keg tapped'),
      ('keg_ended', 'Keg ended'),
  )

  site = models.ForeignKey(KegbotSite, related_name='events')
  kind = models.CharField(max_length=255, choices=KINDS,
      help_text='Type of event.')
  time = models.DateTimeField(help_text='Time of the event.')
  user = models.ForeignKey(User, blank=True, null=True,
      related_name='events',
      help_text='User responsible for the event, if any.')
  drink = models.ForeignKey(Drink, blank=True, null=True,
      related_name='events',
      help_text='Drink involved in the event, if any.')
  keg = models.ForeignKey(Keg, blank=True, null=True,
      related_name='events',
      help_text='Keg involved in the event, if any.')
  session = models.ForeignKey(DrinkingSession, blank=True, null=True,
      related_name='events',
      help_text='Session involved in the event, if any.')

  def __str__(self):
    if self.kind == 'drink_poured':
      ret = 'Drink %i poured' % self.drink.id
    elif self.kind == 'session_started':
      ret = 'Session %s started by drink %s' % (self.session.id,
          self.drink.id)
    elif self.kind == 'session_joined':
      ret = 'Session %s joined by %s (drink %s)' % (self.session.id,
          self.user.username, self.drink.id)
    elif self.kind == 'keg_tapped':
      ret = 'Keg %s tapped' % self.keg.id
    elif self.kind == 'keg_ended':
      ret = 'Keg %s ended' % self.keg.id
    else:
      ret = 'Unknown event type (%s)' % self.kind
    return 'Event %s: %s' % (self.id, ret)

  @classmethod
  def ProcessKeg(cls, keg):
    site = keg.site
    if keg.status == 'online':
      q = keg.events.filter(kind='keg_tapped')
      if q.count() == 0:
        e = keg.events.create(site=site, kind='keg_tapped', time=keg.start_time,
            keg=keg)
        e.save()

    if keg.status == 'offline':
      q = keg.events.filter(kind='keg_ended')
      if q.count() == 0:
        e = keg.events.create(site=site, kind='keg_ended', time=keg.end_time,
            keg=keg)
        e.save()

  @classmethod
  def ProcessDrink(cls, drink):
    keg = drink.keg
    session = drink.session
    site = drink.site
    user = drink.user

    if keg:
      q = keg.events.filter(kind='keg_tapped')
      if q.count() == 0:
        e = keg.events.create(site=site, kind='keg_tapped', time=drink.time,
            keg=keg, user=user, drink=drink, session=session)
        e.save()

    if session:
      q = session.events.filter(kind='session_started')
      if q.count() == 0:
        e = session.events.create(site=site, kind='session_started',
            time=session.start_time, drink=drink, user=user)
        e.save()

    if user:
      q = user.events.filter(kind='session_joined', session=session)
      if q.count() == 0:
        e = user.events.create(site=site, kind='session_joined',
            time=drink.time, session=session, drink=drink, user=user)
        e.save()

    q = drink.events.filter(kind='drink_poured')
    if q.count() == 0:
      e = drink.events.create(site=site, kind='drink_poured',
          time=drink.time, drink=drink, user=user, keg=keg,
          session=session)
      e.save()


def _pics_file_name(instance, filename):
  rand_salt = random.randrange(0xffff)
  new_filename = '%04x-%s' % (rand_salt, filename)
  return os.path.join('pics', new_filename)

class Picture(models.Model):
  site = models.ForeignKey(KegbotSite, related_name='pictures',
      blank=True, null=True,
      help_text='Site owning this picture')
  image = models.ImageField(upload_to=_pics_file_name,
      help_text='The image')
  resized = imagespecs.resized
  thumbnail = imagespecs.thumbnail
  small_resized = imagespecs.small_resized
  small_thumbnail = imagespecs.small_thumbnail

  time = models.DateTimeField(default=timezone.now)

  def __str__(self):
    return 'Picture: %s' % self.image


class PourPicture(models.Model):
  '''Stores additional metadata about a picture taken during a pour.'''
  picture = models.ForeignKey('Picture')
  drink = models.ForeignKey(Drink, blank=True, null=True,
      related_name='pictures',
      help_text='Drink this picture is associated with, if any')
  time = models.DateTimeField(default=timezone.now)
  caption = models.TextField(blank=True, null=True,
      help_text='Caption for the picture')
  user = models.ForeignKey(User, blank=True, null=True,
      help_text='User this picture is associated with, if any')
  keg = models.ForeignKey(Keg, blank=True, null=True, related_name='pictures',
      help_text='Keg this picture is associated with, if any')
  session = models.ForeignKey(DrinkingSession, blank=True, null=True,
      on_delete=models.SET_NULL,
      related_name='pictures',
      help_text='Session this picture is associated with, if any')

  def GetCaption(self):
    if self.caption:
      return self.caption
    elif self.drink:
      if self.user:
        return '%s pouring drink %s' % (self.user.username, self.drink.id)
      else:
        return 'An unknown drinker pouring drink %s' % (self.drink.id,)
    else:
      return ''




# Copyright 2012 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

from django.db import models
from pykeg.core import models as core_models
from socialregistration import signals as sr_signals
from socialregistration.contrib.foursquare.models import FoursquareProfile

### Site-specific

class SiteFoursquareSettings(models.Model):
  site = models.OneToOneField(core_models.KegbotSite, unique=True,
      related_name='foursquare_settings')
  venue_id = models.CharField(max_length=256,
      help_text='The foursquare venue id for this Kegbot.')
  venue_name = models.CharField(max_length=256,
      help_text='The foursquare venue name for this Kegbot.')

### User-specific

class FoursquareSettings(models.Model):
  profile = models.OneToOneField(FoursquareProfile, unique=True,
      related_name='settings')
  enabled = models.BooleanField(default=True,
      help_text='Deselect to disable all foursquare activity.')
  checkin_session_joined = models.BooleanField(default=True,
      help_text='Checkin when I start drinking.')

def _foursquare_connect_handler(sender, user, profile, client, request=None, **kwargs):
  settings, _ = FoursquareSettings.objects.get_or_create(profile=profile)
  settings.save()

sr_signals.connect.connect(_foursquare_connect_handler, sender=FoursquareProfile,
    dispatch_uid = 'foursquare.connect')




# Copyright 2003-2009 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

import uuid

from kegbot.util import units

from . import models

def db_is_installed():
  return len(models.KegbotSite.objects.all()) > 0

class AlreadyInstalledError(Exception):
  """Thrown when database is already installed."""

def set_defaults(force=False):
  """ default values (contents may change with schema) """
  if not force and db_is_installed():
    raise AlreadyInstalledError("Database is already installed.")

  site = models.KegbotSite.objects.create(name='default', is_setup=False)

  # KegTap defaults
  main_tap = models.KegTap(site=site, name='Main Tap', meter_name='kegboard.flow0')
  main_tap.save()
  secondary_tap = models.KegTap(site=site, name='Second Tap', meter_name='kegboard.flow1')
  secondary_tap.save()

  # brewer defaults
  unk_brewer = models.Brewer(name='Unknown Brewer')
  unk_brewer.save()

  # beerstyle defaults
  unk_style = models.BeerStyle(name='Unknown Style')
  unk_style.save()

  # beertype defaults
  unk_type = models.BeerType(name="Unknown Beer", brewer=unk_brewer, style=unk_style)
  unk_type.save()

  # KegSize defaults - from http://en.wikipedia.org/wiki/Keg#Size
  default_sizes = (
    (15.5, "Full Keg (half barrel)"),
    (13.2, "Import Keg (European barrel)"),
    (7.75, "Pony Keg (quarter barrel)"),
    (6.6, "European Half Barrel"),
    (5.23, "Sixth Barrel (torpedo keg)"),
    (5.0, "Corny Keg"),
    (1.0, "Mini Keg"),
  )
  for gallons, description in default_sizes:
    volume = units.Quantity(gallons, units.UNITS.USGallon)
    volume_int = volume.Amount(in_units=units.RECORD_UNIT)

    ks = models.KegSize(
      name=description,
      volume_ml=volume_int,
    )
    ks.save()



from django.conf.urls import include
from django.conf.urls import patterns
from django.conf.urls import url

urlpatterns = patterns('pykeg.web.kegadmin.views',
    ### main page
    url(r'^$', 'dashboard', name='kegadmin-dashboard'),
    url(r'^settings/$', 'general_settings', name='kegadmin-main'),

    url(r'^taps/$', 'tap_list', name='kegadmin-taps'),
    url(r'^taps/create/$', 'add_tap', name='kegadmin-add-tap'),
    url(r'^taps/(?P<tap_id>\d+)/$', 'tap_detail', name='kegadmin-edit-tap'),

    url(r'^users/$', 'user_list', name='kegadmin-users'),
    url(r'^users/create/$', 'add_user', name='kegadmin-add-user'),
    url(r'^users/(?P<user_id>\d+)/$', 'user_detail', name='kegadmin-edit-user'),

    url(r'^tokens/$', 'token_list', name='kegadmin-tokens'),
    url(r'^tokens/create/$', 'add_token', name='kegadmin-add-token'),
    url(r'^tokens/(?P<token_id>\d+)/$', 'token_detail', name='kegadmin-edit-token'),

    url(r'^backup-restore/$', 'backup_restore', name='kegadmin-backup-restore'),
    url(r'^backup-restore/dump/$', 'generate_backup', name='kegadmin-get-backup'),
    url(r'^connections/', include('pykeg.connections.urls')),
    url(r'^edit-connections/$', 'connections', name='kegadmin-connections'),
    url(r'^logs/$', 'logs', name='kegadmin-logs'),
    url(r'^autocomplete/beer/$', 'autocomplete_beer_type',
      name='kegadmin-autocomplete-beer'),
    url(r'^autocomplete/user/$', 'autocomplete_user',
      name='kegadmin-autocomplete-user'),
    url(r'^autocomplete/token/$', 'autocomplete_token',
      name='kegadmin-autocomplete-token'),
)

urlpatterns += patterns('pykeg.connections.twitter.views',
    url('^redirect/$', 'site_twitter_redirect', name='site_twitter_redirect'),
    url('^callback/$', 'site_twitter_callback', name='site_twitter_callback'),
)



# Copyright 2010 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Routines from converting data to and from Protocol Buffer format."""

import pytz

from django.conf import settings

from kegbot.api import api_pb2
from kegbot.api import models_pb2
from kegbot.api import protoutil
from kegbot.util import util

from pykeg.contrib.soundserver import models as soundserver_models
from pykeg.core import models

_CONVERSION_MAP = {}

def converts(kind):
  def decorate(f):
    global _CONVERSION_MAP
    _CONVERSION_MAP[kind] = f
    return f
  return decorate

def datestr(dt):
  if settings.USE_TZ:
    return dt.isoformat()
  try:
    # Convert from local to UTC.
    # TODO(mikey): handle incoming datetimes with tzinfo.
    dt = util.local_to_utc(dt, settings.TIME_ZONE)
  except pytz.UnknownTimeZoneError:
    pass
  return dt.strftime('%Y-%m-%dT%H:%M:%SZ')

def ToProto(obj, full=False):
  """Converts the object to protocol format."""
  if obj is None:
    return None
  kind = obj.__class__
  if hasattr(obj, '__iter__'):
    return [ToProto(item, full) for item in obj]
  elif kind in _CONVERSION_MAP:
    return _CONVERSION_MAP[kind](obj, full)
  else:
    raise ValueError, "Unknown object type: %s" % kind

def ToDict(obj, full=False):
  res = ToProto(obj, full)
  if hasattr(res, '__iter__'):
    return [protoutil.ProtoMessageToDict(m) for m in res]
  else:
    return protoutil.ProtoMessageToDict(res)

### Model conversions

@converts(models.AuthenticationToken)
def AuthTokenToProto(record, full=False):
  ret = models_pb2.AuthenticationToken()
  ret.id = record.id
  ret.auth_device = record.auth_device
  ret.token_value = record.token_value
  if record.user:
    ret.username = str(record.user.username)
    ret.user.MergeFrom(ToProto(record.user))
  if record.nice_name:
    ret.nice_name = record.nice_name
  ret.created_time = datestr(record.created_time)
  ret.enabled = record.enabled
  if record.expire_time:
    ret.expire_time = datestr(record.expire_time)
  if record.pin:
    ret.pin = record.pin
  return ret

@converts(models.Picture)
def PictureToProto(record, full=False):
  ret = models_pb2.Image()
  ret.url = record.resized.url
  ret.original_url = record.image.url
  ret.thumbnail_url = record.thumbnail.url
  # TODO(mikey): This can be expensive depending on the storage backend
  # (attempts to fetch image).
  #try:
  #  ret.width = record.image.width
  #  ret.height = record.image.height
  #except IOError:
  #  pass
  return ret

@converts(models.PourPicture)
def PourPictureToProto(record, full=False):
  ret = PictureToProto(record.picture)
  if record.time:
    ret.time = datestr(record.time)
  if record.caption:
    ret.caption = record.caption
  if record.user:
    ret.user_id = record.user.username
  if record.keg:
    ret.keg_id = record.keg.id
  if record.session:
    ret.session_id = record.session.id
  if record.drink:
    ret.drink_id = record.drink.id
  return ret

@converts(models.BeerStyle)
def BeerStyleToProto(style, full=False):
  ret = models_pb2.BeerStyle()
  ret.id = str(style.id)
  ret.name = style.name
  return ret

@converts(models.BeerType)
def BeerTypeToProto(beertype, full=False):
  ret = models_pb2.BeerType()
  ret.id = str(beertype.id)
  ret.name = beertype.name
  ret.brewer_id = str(beertype.brewer.id)
  ret.style_id = str(beertype.style.id)
  if beertype.edition is not None:
    ret.edition = beertype.edition
  # TODO(mikey): guarantee this at DB level
  abv = beertype.abv or 0.0
  ret.abv = max(min(abv, 100.0), 0.0)
  if beertype.calories_oz is not None:
    ret.calories_oz = beertype.calories_oz
  if beertype.carbs_oz is not None:
    ret.carbs_oz = beertype.carbs_oz
  if beertype.specific_gravity is not None:
    ret.specific_gravity = beertype.specific_gravity
  if beertype.original_gravity is not None:
    ret.original_gravity = beertype.original_gravity
  if beertype.image:
    ret.image.MergeFrom(ToProto(beertype.image))
  return ret

@converts(models.Brewer)
def BrewerToProto(brewer, full=False):
  ret = models_pb2.Brewer()
  ret.id = str(brewer.id)
  ret.name = brewer.name
  if brewer.country is not None:
    ret.country = brewer.country
  if brewer.origin_state is not None:
    ret.origin_state = brewer.origin_state
  if brewer.origin_city is not None:
    ret.origin_city = brewer.origin_city
  if brewer.production is not None:
    ret.production = brewer.production
  if brewer.url is not None:
    ret.url = brewer.url
  if brewer.description is not None:
    ret.description = brewer.description
  if brewer.image:
    ret.image.MergeFrom(ToProto(brewer.image))
  return ret

@converts(models.Drink)
def DrinkToProto(drink, full=False):
  ret = models_pb2.Drink()
  ret.id = drink.id
  ret.url = drink.get_absolute_url()
  ret.ticks = drink.ticks
  ret.volume_ml = drink.volume_ml
  ret.session_id = drink.session.id
  ret.time = datestr(drink.time)
  ret.duration = drink.duration
  ret.status = drink.status
  if drink.keg:
    ret.keg_id = drink.keg.id
  if drink.user:
    ret.user_id = drink.user.username
  if drink.shout:
    ret.shout = drink.shout
  if drink.tick_time_series:
    ret.tick_time_series = drink.tick_time_series

  if full:
    if drink.user:
      ret.user.MergeFrom(ToProto(drink.user))
    if drink.keg:
      ret.keg.MergeFrom(ToProto(drink.keg))
    if drink.session:
      ret.session.MergeFrom(ToProto(drink.session))
    for i in drink.pictures.all():
      ret.images.add().MergeFrom(ToProto(i))
  return ret

@converts(models.Keg)
def KegToProto(keg, full=False):
  ret = models_pb2.Keg()
  ret.id = keg.id
  ret.url = keg.get_absolute_url()
  ret.type_id = str(keg.type.id)
  ret.size_id = keg.size.id
  ret.size_name = keg.size.name
  ret.size_volume_ml = keg.size.volume_ml
  rem = float(keg.remaining_volume())
  ret.volume_ml_remain = rem
  ret.percent_full = keg.percent_full()
  ret.start_time = datestr(keg.start_time)
  ret.end_time = datestr(keg.end_time)
  ret.status = keg.status
  if keg.description is not None:
    ret.description = keg.description
  ret.spilled_ml = keg.spilled_ml

  if full:
    if keg.type:
      ret.type.MergeFrom(ToProto(keg.type))
    if keg.size:
      ret.size.MergeFrom(ToProto(keg.size))

  return ret

@converts(models.KegSize)
def KegSizeToProto(size, full=False):
  ret = models_pb2.KegSize()
  ret.id = size.id
  ret.name = size.name
  ret.volume_ml = size.volume_ml
  return ret

@converts(models.KegTap)
def KegTapToProto(tap, full=False):
  ret = models_pb2.KegTap()
  ret.id = tap.id
  ret.name = tap.name
  ret.meter_name = tap.meter_name
  ret.relay_name = tap.relay_name or ''
  ret.ml_per_tick = tap.ml_per_tick
  if tap.description is not None:
    ret.description = tap.description
  if tap.current_keg:
    ret.current_keg_id = tap.current_keg.id
    if full:
      ret.current_keg.MergeFrom(ToProto(tap.current_keg, full=True))

  if tap.temperature_sensor:
    ret.thermo_sensor_id = tap.temperature_sensor.id
    log = tap.temperature_sensor.LastLog()
    if log:
      ret.last_temperature.MergeFrom(ToProto(log))
  return ret

@converts(models.DrinkingSession)
def SessionToProto(record, full=False):
  ret = models_pb2.Session()
  ret.id = record.id
  ret.url = record.get_absolute_url()
  ret.start_time = datestr(record.start_time)
  ret.end_time = datestr(record.end_time)
  ret.volume_ml = record.volume_ml
  ret.name = record.name or ''

  if full:
    #ret.stats.MergeFrom(record.GetStats())
    ret.is_active = record.IsActiveNow()
  return ret

@converts(models.Thermolog)
def ThermoLogToProto(record, full=False):
  ret = models_pb2.ThermoLog()
  ret.id = record.id
  ret.sensor_id = record.sensor.id
  ret.temperature_c = record.temp
  ret.time = datestr(record.time)
  return ret

@converts(models.ThermoSensor)
def ThermoSensorToProto(record, full=False):
  ret = models_pb2.ThermoSensor()
  ret.id = record.id
  ret.sensor_name = record.raw_name
  ret.nice_name = record.nice_name
  return ret

@converts(models.User)
def UserToProto(user, full=False):
  ret = models_pb2.User()
  ret.username = user.username
  ret.url = user.get_profile().get_absolute_url()
  ret.is_active = user.is_active
  if full:
    ret.first_name = user.first_name
    ret.last_name = user.last_name
    ret.email = user.email
    ret.is_staff = user.is_staff
    ret.is_active = user.is_active
    ret.is_superuser = user.is_superuser
    ret.last_login = datestr(user.last_login)
    ret.date_joined = datestr(user.date_joined)
  profile = user.get_profile()
  if profile.mugshot:
    ret.image.MergeFrom(ToProto(profile.mugshot))
  return ret

@converts(models.UserProfile)
def UserProfileToProto(record, full=False):
  ret = models_pb2.UserProfile()
  ret.username = record.user.username
  return ret

@converts(models.SystemStats)
@converts(models.UserStats)
@converts(models.KegStats)
@converts(models.SessionStats)
def SystemStatsToProto(record, full=False):
  return protoutil.DictToProtoMessage(record.stats, models_pb2.Stats())

@converts(models.SystemEvent)
def SystemEventToProto(record, full=False):
  ret = models_pb2.SystemEvent()
  ret.id = record.id
  ret.kind = record.kind
  ret.time = datestr(record.time)

  if record.drink:
    ret.drink_id = record.drink.id
    if full:
      ret.drink.MergeFrom(ToProto(record.drink, full=True))
  if record.keg:
    ret.keg_id = record.keg.id
    if full:
      ret.keg.MergeFrom(ToProto(record.keg, full=True))
  if record.session:
    ret.session_id = record.session.id
    if full:
      ret.session.MergeFrom(ToProto(record.session, full=True))
  if record.user:
    ret.user_id = str(record.user.username)
    if full:
      ret.user.MergeFrom(ToProto(record.user, full=True))

  image = None
  if record.kind in ('drink_poured', 'session_started', 'session_joined') and record.user:
    image = record.user.get_profile().mugshot
  elif record.kind in ('keg_tapped', 'keg_ended'):
    if record.keg.type and record.keg.type.image:
      image = record.keg.type.image
  if image:
    ret.image.MergeFrom(ToProto(image))

  return ret

@converts(soundserver_models.SoundEvent)
def SoundEventToProto(record, full=False):
  ret = models_pb2.SoundEvent()
  ret.event_name = record.event_name
  ret.event_predicate = record.event_predicate
  ret.sound_url = record.soundfile.sound.url
  if record.user:
    ret.user = record.user
  return ret

# Composite messages

def GetDrinkDetail(drink):
  ret = api_pb2.DrinkDetail()
  ret.drink.MergeFrom(ToProto(drink))
  if drink.user:
    ret.user.MergeFrom(ToProto(drink.user))
  if drink.keg:
    ret.keg.MergeFrom(ToProto(drink.keg))
  if drink.session:
    ret.session.MergeFrom(ToProto(drink.session))
  for i in drink.pictures.all():
    ret.images.add().MergeFrom(ToProto(i))
  return ret




# Copyright 2010 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Migrate timestamps.

Prior to Kegbot 0.9.8, timestamps were stored in the database in localtime. This
tool adjusts these timestamps to UTC time.

You should only use this command if you are upgrading from a version of Kegbot
earlier than 0.9.8.  If all of your data was generated by Kegbot 0.9.8 or newer,
or if you have already run this command, you should NOT proceed.

WARNING: Back up your data before proceeding.
"""

HELP_TEXT = __doc__

import pytz
import sys

from django.db import transaction
from django.conf import settings
from django.core.management.base import CommandError
from django.core.management.base import NoArgsCommand

from django.utils import timezone
from pykeg.core import models

class Command(NoArgsCommand):
  help = u'Regenerates timestamps due to timezone conversion.'
  args = '<none>'

  def handle(self, **options):
    print HELP_TEXT
    confirm('Press Y to continue. ')

    print 'Please enter the SESSION ID for the LAST session you want to '
    print 'migrate.  All drinks UP TO AND INCLUDING this session will be '
    print 'migrated. '

    default_session = models.DrinkingSession.objects.all().order_by('-id')[0]
    lbl = '%s (%s)' % (default_session.id, default_session.start_time)

    print ''
    session_id = raw_input('Session id [%s]: ' % lbl).strip()
    print ''
    if not session_id:
      session_id = default_session.id
    else:
      session_id = int(session_id)

    session = models.DrinkingSession.objects.get(id=session_id)

    print ''
    print 'Your current settings.TIME_ZONE is: %s' % (settings.TIME_ZONE,)
    print 'Drinks will be migrated assuming they occurred in this zone. '
    print ''
    confirm('Correct time zone?')

    print ''
    print 'You selected session id %s.' % (session.id,)
    print ''
    print '   Current start time: %s' % timezone.localtime(session.start_time)
    print ' Corrected start time: %s' % timezone.localtime(convert(session.start_time))
    print '           Difference: %s' % (convert(session.start_time) - session.start_time)

    print ''
    confirm('Does this look correct?')
    print ''

    drinks = models.Drink.objects.filter(session_id__lte=session.id).order_by('id')
    print 'There are %s drinks up to and including this session.' % len(drinks)

    print ''
    confirm('Start migration?')
    print ''

    with transaction.commit_on_success():
      do_migrate(drinks)

      print ''
      print 'All items migrated.  Please spot check the report that just '
      print 'scrolled by.  If in doubt, abort now and no changes will be '
      print 'made.'
      print ''
      print 'WARNING: Pressing Y will permanently save adjusted times.'
      print ''
      confirm('Commit transaction?')
      print ''
      print 'Committing transaction..'
      print ''

    print 'Migration finished.'

def migrate(obj, attrs, errors):
  print '  %s' % str(obj)
  for attr in attrs:
    old = getattr(obj, attr)
    if old:
      try:
        new = convert(old)
      except pytz.exceptions.NonExistentTimeError, e:
        print '    - ERR: %s' % e
        errors.append((obj, attr, e))
        continue
      setattr(obj, attr, new)
      print '    - %s: %s -> %s' % (attr, old, new)
  obj.save()


def do_migrate(drinks):
  LT = timezone.localtime

  errors = []
  kegs = set()
  sessions = set()
  pictures = []
  for drink in drinks:
    migrate(drink, ['time'], errors)
    if drink.keg:
      kegs.add(drink.keg)
    sessions.add(drink.session)
    for p in drink.pictures.all().order_by('id'):
      pictures.append(p)

  print ''
  for picture in pictures:
    migrate(picture, ['time'], errors)

  print ''
  for session in sessions:
    migrate(session, ['start_time', 'end_time'], errors)

  print ''
  for keg in kegs:
    migrate(keg, ['start_time', 'end_time'], errors)

  if errors:
    print ''
    print 'ERROR: MIGRATION ABORTED'
    print ''
    print 'The following objects have impossibe/non-existent times.'
    print 'Please fix them manually and re-run.'
    print ''
    for obj, attr, e in errors:
      print '  %s: %s: %s' % (obj, attr, e)
    print ''
    raise ValueError('Invalid time(s) found.')


def convert(dt):
  return timezone.make_aware(timezone.make_naive(dt, pytz.UTC),
      pytz.timezone(settings.TIME_ZONE))


def confirm(prompt):
  val = raw_input('%s [y/N]: ' % prompt)
  if not val or val[0].lower() != 'y':
    raise ValueError('Aborted by user.')
  print ''




# Pykeg main settings file.

# Note: YOU SHOULD NOT NEED TO EDIT THIS FILE.  Instead, see the instructions in
# local_settings.py.example.

# Grab flags for optional modules.
from pykeg.core.optional_modules import *

INSTALLED_APPS = (
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.humanize',
    'django.contrib.markup',
    'django.contrib.messages',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.staticfiles',

    'crispy_forms',
    'bootstrap-pagination',
    'imagekit',
    'pykeg.connections',
    'pykeg.connections.foursquare',
    'pykeg.connections.twitter',
    'pykeg.connections.untappd',
    'pykeg.contrib.soundserver',
    'pykeg.core',
    'pykeg.web',
    'pykeg.web.api',
    'pykeg.web.account',
    'pykeg.web.charts',
    'pykeg.web.kegadmin',
    'pykeg.web.kegweb',
    'pykeg.web.setup_wizard',
    'gunicorn',
    'icanhaz',
    'registration',
    'socialregistration',
    'socialregistration.contrib.twitter',
    'socialregistration.contrib.facebook',
    'socialregistration.contrib.foursquare',
    'south',
    'django_nose', # must be after south
)

AUTH_PROFILE_MODULE = "core.UserProfile"
LOGIN_REDIRECT_URL = "/account/"

# List of finder classes that know how to find static files in
# various locations.
STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
)

### Kegweb specific stuff

ROOT_URLCONF = 'pykeg.web.urls'

TEMPLATE_DIRS = (
    'web/templates',
)

SITE_ID = 1

# Language code for this installation. All choices can be found here:
# http://www.w3.org/TR/REC-html40/struct/dirlang.html#langcodes
# http://blogs.law.harvard.edu/tech/stories/storyReader$15
LANGUAGE_CODE = 'en-us'

# Local time zone for this installation. All choices can be found here:
# http://www.postgresql.org/docs/8.1/static/datetime-keywords.html#DATETIME-TIMEZONE-SET-TABLE
TIME_ZONE = 'America/Los_Angeles'

# Enable Django 1.4+ timezone support.  Do not disable this.
USE_TZ = True

# Absolute filesystem path to the directory that will hold user-uploaded files.
# Example: "/home/media/media.lawrence.com/media/"
MEDIA_ROOT = ''

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash.
# Examples: "http://media.lawrence.com/media/", "http://example.com/media/"
MEDIA_URL = '/media/'

# Absolute path to the directory static files should be collected to.
# Don't put anything in this directory yourself; store your static files
# in apps' "static/" subdirectories and in STATICFILES_DIRS.
# Example: "/home/media/media.lawrence.com/static/"
STATIC_ROOT = ''

# URL prefix for static files.
# Example: "http://media.lawrence.com/static/"
STATIC_URL = '/static/'

# URL prefix for admin static files -- CSS, JavaScript and images.
# Make sure to use a trailing slash.
# Examples: "http://foo.com/static/admin/", "/static/admin/".
ADMIN_MEDIA_PREFIX = '/static/admin/'

# Disable Django's built in host checker.
# see pykeg.web.middleware.HttpHostMiddleware
ALLOWED_HOSTS = ['*']

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
    'django.template.loaders.eggs.Loader',
)

TEMPLATE_CONTEXT_PROCESSORS = (
    'django.core.context_processors.debug',
    'django.core.context_processors.i18n',
    'django.core.context_processors.media',
    'django.core.context_processors.static',
    'django.core.context_processors.request',
    'django.contrib.auth.context_processors.auth',
    'django.contrib.messages.context_processors.messages',
    'pykeg.web.context_processors.enabled_features',
    'pykeg.web.context_processors.kbsite',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.cache.UpdateCacheMiddleware',
    'django.middleware.gzip.GZipMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.middleware.transaction.TransactionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',

    'pykeg.web.middleware.KegbotSiteMiddleware',
    'pykeg.web.middleware.HttpHostMiddleware',
    'pykeg.web.middleware.SiteActiveMiddleware',
    'pykeg.web.api.middleware.ApiRequestMiddleware',
    'pykeg.web.middleware.PrivacyMiddleware',

    'django.middleware.doc.XViewMiddleware',

    # Cache middleware should be last, except for ApiResponseMiddleWare,
    # which needs to be after it (in request order) so that it can
    # update the Cache-Control header before it (in reponse order).
    'django.middleware.cache.FetchFromCacheMiddleware',
    'pykeg.web.api.middleware.ApiResponseMiddleware',
)

AUTHENTICATION_BACKENDS = (
    'django.contrib.auth.backends.ModelBackend',
)

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
        'LOCATION': 'cache',
    }
}

CACHE_MIDDLEWARE_ANONYMOUS_ONLY = True

INTERNAL_IPS = ('127.0.0.1',)

### Celery
if HAVE_DJCELERY and HAVE_DJKOMBU:
  INSTALLED_APPS += (
    'djcelery',
    'djkombu',
  )

  import djcelery
  djcelery.setup_loader()
  BROKER_URL = "django://"

  CELERY_QUEUES = {
    'default' : {
      'exchange': 'default',
      'binding_key': 'default'
    },
  }
  CELERY_DEFAULT_QUEUE = "default"
  CELERYD_CONCURRENCY = 3

### debug_toolbar

if HAVE_DEBUG_TOOLBAR:
  INSTALLED_APPS += (
    'debug_toolbar',
  )
  MIDDLEWARE_CLASSES += (
    'debug_toolbar.middleware.DebugToolbarMiddleware',
  )
  DEBUG_TOOLBAR_PANELS = (
      'debug_toolbar.panels.version.VersionDebugPanel',
      'debug_toolbar.panels.timer.TimerDebugPanel',
      'debug_toolbar.panels.settings_vars.SettingsVarsDebugPanel',
      'debug_toolbar.panels.headers.HeaderDebugPanel',
      'debug_toolbar.panels.request_vars.RequestVarsDebugPanel',
      'debug_toolbar.panels.template.TemplateDebugPanel',
      'debug_toolbar.panels.sql.SQLDebugPanel',
      'debug_toolbar.panels.signals.SignalDebugPanel',
      'debug_toolbar.panels.logger.LoggingPanel',
      #'debug_toolbar.panels.profiling.ProfilingDebugPanel',
  )

### logging

LOGGING = {
  'version': 1,
  'disable_existing_loggers': True,
  'root': {
    'level': 'WARNING',
    'handlers': ['console'],  # replaced with sentry later, if available.
  },
  'filters': {
    'require_debug_true': {
      '()': 'pykeg.core.logger.RequireDebugTrue',
    },
  },
  'handlers': {
    'console': {
      'level': 'DEBUG',
      'filters': ['require_debug_true'],
      'class': 'logging.StreamHandler',
    },
    'null': {
      'class': 'django.utils.log.NullHandler',
    },
    'cache': {
      'level': 'ERROR',
      'class': 'pykeg.core.logger.CacheHandler',
    },
  },
  'loggers': {
    'django': {
      'handlers': ['console', 'cache'],
    },
    'pykeg': {
      'handlers': ['console', 'cache'],
    },
    'raven': {
      'level': 'DEBUG',
      'handlers': ['console'],
      'propagate': False,
    },
    'sentry.errors': {
      'level': 'DEBUG',
      'handlers': ['console'],
      'propagate': False,
    },
  },
}

### raven

if HAVE_RAVEN:
  INSTALLED_APPS += (
    'raven.contrib.django',
  )

  LOGGING['root']['handlers'] = ['sentry']
  LOGGING['handlers']['sentry'] = {
    'level': 'ERROR',
    'class': 'raven.contrib.django.handlers.SentryHandler',
  }

### sentry

if HAVE_SENTRY:
  INSTALLED_APPS += (
    'sentry',
  )

### tornado

if HAVE_DJANGOTORNADO:
  INSTALLED_APPS += (
    'rjdj.djangotornado',
  )


### django.contrib.messages
MESSAGE_STORAGE = 'django.contrib.messages.storage.fallback.FallbackStorage'

### django-registration
ACCOUNT_ACTIVATION_DAYS = 3

### E-mail
EMAIL_BACKEND = 'django.core.mail.backends.dummy.EmailBackend'

# Bogus default values (to prevent djangofb from choking if unavailable);
# replace with site-specific values in local_settings.py, if desired.
FACEBOOK_API_KEY = ''
FACEBOOK_SECRET_KEY = ''

### Twitter

TWITTER_CONSUMER_KEY = ''
TWITTER_CONSUMER_SECRET_KEY =''
TWITTER_REQUEST_TOKEN_URL = 'https://api.twitter.com/oauth/request_token'
TWITTER_ACCESS_TOKEN_URL = 'https://api.twitter.com/oauth/access_token'
TWITTER_AUTHORIZATION_URL = 'https://api.twitter.com/oauth/authorize'

### Foursquare

FOURSQUARE_CLIENT_ID = ''
FOURSQUARE_CLIENT_SECRET = ''
FOURSQUARE_REQUEST_PERMISSIONS = ''

### Untappd

UNTAPPD_CLIENT_ID = ''
UNTAPPD_CLIENT_SECRET = ''

TEST_RUNNER = 'django_nose.NoseTestSuiteRunner'
NOSE_ARGS = ['--exe']
SKIP_SOUTH_TESTS = True
SOUTH_TESTS_MIGRATE = False

ICANHAZ_APP_DIRNAMES = ['static/jstemplates', 'jstemplates']

from pykeg.core import importhacks
try:
  from local_settings import *
except ImportError:
  try:
    from common_settings import *
    import warnings
    warnings.warn('common_settings.py should be renamed to local_settings.py')
  except ImportError:
    msg = """ Error: Could not find local_settings.py
    Most likely, this means Kegbot has not been configured properly.
    Consult setup documentation.  Exiting..."""
    raise ImportError(msg)

### socialregistration (after importing common settings)
if FACEBOOK_API_KEY and FACEBOOK_SECRET_KEY:
  MIDDLEWARE_CLASSES += (
    'socialregistration.middleware.FacebookMiddleware',
  )
  AUTHENTICATION_BACKENDS += (
    'socialregistration.contrib.facebook.auth.FacebookAuth',
  )

if TWITTER_CONSUMER_KEY and TWITTER_CONSUMER_SECRET_KEY:
  AUTHENTICATION_BACKENDS += (
    'socialregistration.contrib.twitter.auth.TwitterAuth',
  )



# Copyright 2003-2011 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

from django.utils import unittest

from kegbot.api import models_pb2
from kegbot.api.protoutil import ProtoMessageToDict

from . import backend
from . import models
from . import stats
from .testutils import make_datetime

class StatsTestCase(unittest.TestCase):
  def setUp(self):
    self.site, created = models.KegbotSite.objects.get_or_create(name='default')
    self.backend = backend.KegbotBackend(site=self.site)

    test_usernames = ('user1', 'user2', 'user3')
    self.users = [self.backend.CreateNewUser(name) for name in test_usernames]

    self.taps = [
        self.backend.CreateTap('tap1', 'kegboard.flow0', ml_per_tick=1/2200.0),
        self.backend.CreateTap('tap2', 'kegboard.flow1', ml_per_tick=1/2200.0),
    ]

    #self.drinks = []
    #for user in self.users:
    #  for amt in (100, 200, 300):
    #    d = self.backend.RecordDrink('kegboard.flow0', ticks=amt,
    #        volume_ml=amt, username=user.username, do_postprocess=False)
    #    self.drinks.append(d)

  def assertProtosEqual(self, expected, actual):
    d1 = ProtoMessageToDict(expected)
    d2 = ProtoMessageToDict(actual)
    msg = ''
    for k, v in d1.iteritems():
      if k not in d2:
        msg += 'Value for %s not present in actual. \n' % k
      elif v != d2[k]:
        msg += 'Values for %s differ: expected "%s", actual "%s". \n' % (k, v, d2[k])
    for k, v in d2.iteritems():
      if k not in d1:
        msg += 'Value for %s not present in expected. \n' % k
    if msg:
      self.fail(msg)

  def _getEmptyStats(self):
    s = models_pb2.Stats()
    s.last_drink_id = 0
    s.total_volume_ml = 0.0
    s.total_pours = 0
    s.average_volume_ml = 0.0
    s.greatest_volume_ml = 0.0
    s.greatest_volume_id = 0
    return s

  def testStuff(self):
    builder = stats.SystemStatsBuilder(None)

    empty_stats = models_pb2.Stats()
    system_stats_d0 = builder.Build()
    self.assertEquals(empty_stats, system_stats_d0)

    # Record a new drink and verify stats.
    pour_time = make_datetime(2011, 05, 01, 12, 00)
    self.backend.RecordDrink('kegboard.flow0', ticks=100,
        volume_ml=100, username='user1', pour_time=pour_time,
        do_postprocess=False)
    drink1 = models.Drink.objects.get(id=1)

    expected = self._getEmptyStats()
    expected.last_drink_id = 1
    expected.total_volume_ml = 100.0
    expected.total_pours = 1
    expected.average_volume_ml = 100.0
    expected.greatest_volume_ml = 100.0
    expected.greatest_volume_id = 1
    expected.has_guest_pour = False
    d = expected.volume_by_day_of_week.add()
    d.weekday = "0"  # Sunday
    d.volume_ml = 100
    u = expected.volume_by_drinker.add()
    u.username = "user1"
    u.volume_ml = 100
    y = expected.volume_by_year.add()
    y.year = 2011
    y.volume_ml = 100
    expected.registered_drinkers.append("user1")
    expected.sessions_count = 1

    system_stats_d1 = stats.SystemStatsBuilder(drink1).Build()
    self.assertProtosEqual(expected, system_stats_d1)

    # Pour another drink
    self.backend.RecordDrink('kegboard.flow0', ticks=200,
        volume_ml=200, username='user1', pour_time=pour_time,
        do_postprocess=False)
    drink2 = models.Drink.objects.get(id=2)

    # Adjust stats
    expected.last_drink_id = 2
    expected.total_volume_ml = 300.0
    expected.total_pours = 2
    expected.average_volume_ml = 150.0
    expected.greatest_volume_ml = 200.0
    expected.greatest_volume_id = 2
    d = expected.volume_by_day_of_week[0]
    d.volume_ml += 200
    u = expected.volume_by_drinker[0]
    u.volume_ml += 200
    y = expected.volume_by_year[0]
    y.volume_ml += 200

    system_stats_d2 = stats.SystemStatsBuilder(drink2).Build()
    self.assertProtosEqual(expected, system_stats_d2)

    # Build the same stats incrementally and verify identical result.
    system_stats_d2_inc = stats.SystemStatsBuilder(drink2, system_stats_d1).Build()
    self.assertProtosEqual(system_stats_d2, system_stats_d2_inc)




from django.conf.urls import include
from django.conf.urls import patterns
from django.conf.urls import url

urlpatterns = patterns('',
    url('^twitter/', include('pykeg.connections.twitter.urls')),
)




from django.conf import settings

import pykeg
from pykeg.core import features
from pykeg.core import models
from pykeg.web.kegweb.forms import LoginForm

def enabled_features(request):
  """Adds a USE_FEATURENAME flags for each enabled feature (see features.py)"""
  # TODO(mikey): this might make it harder to diagnose why features aren't
  # visible/being used.
  ret = {}
  ret['USE_FACEBOOK'] = features.use_facebook()
  ret['USE_FOURSQUARE'] = features.use_foursquare()
  ret['USE_TWITTER'] = features.use_twitter()
  ret['USE_UNTAPPD'] = features.use_untappd()
  return ret

def kbsite(request):
  kbsite = getattr(request, 'kbsite', None)
  analytics_id = None
  if kbsite:
    analytics_id = kbsite.settings.google_analytics_id

  guest_info = {
    'name': 'guest',
    'image': None,
  }

  serial = ''
  if kbsite:
    guest_info['name'] = kbsite.settings.guest_name
    guest_info['image'] = kbsite.settings.guest_image
    serial = kbsite.serial_number

  ret = {
    'DEBUG': settings.DEBUG,
    'EPOCH': pykeg.EPOCH,
    'VERSION': pykeg.__version__,
    'SERIAL_NUMBER': serial,
    'kbsite': getattr(request, 'kbsite', None),
    'request_path': request.path,
    'login_form': LoginForm(initial={'next_page': request.path}),
    'GOOGLE_ANALYTICS_ID': analytics_id,
    'guest_info': guest_info,
  }
  return ret



# Copyright 2013 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""High-level interface to Django backend."""

from __future__ import absolute_import

import datetime
import logging

from django.conf import settings
from django.utils import timezone
from . import kb_common
from . import models
from . import time_series

if settings.HAVE_CELERY:
  from pykeg.web import tasks

class BackendError(Exception):
  """Base backend error exception."""

class NoTokenError(BackendError):
  """Token given is unknown."""

class KegbotBackend:
  """Django Backend."""

  def __init__(self, sitename='default', site=None):
    self._logger = logging.getLogger('backend')
    if site:
      self._site = site
    else:
      self._site = models.KegbotSite.objects.get(name=sitename)

  def _GetTapFromName(self, tap_name):
    try:
      return models.KegTap.objects.get(site=self._site, meter_name=tap_name)
    except models.KegTap.DoesNotExist:
      return None

  def _GetKegForTapName(self, tap_name):
    tap = self._GetTapFromName(tap_name)
    if tap and tap.current_keg and tap.current_keg.status == 'online':
      return tap.current_keg
    return None

  def _GetSensorFromName(self, name, autocreate=True):
    try:
      return models.ThermoSensor.objects.get(site=self._site, raw_name=name)
    except models.ThermoSensor.DoesNotExist:
      if autocreate:
        sensor = models.ThermoSensor(site=self._site, raw_name=name, nice_name=name)
        sensor.save()
        return sensor
      else:
        return None

  def _GetUserObjFromUsername(self, username):
    try:
      return models.User.objects.get(username=username)
    except models.User.DoesNotExist:
      return None

  def CreateNewUser(self, username):
    return models.User.objects.create(username=username)

  def CreateTap(self, name, meter_name, relay_name=None, ml_per_tick=None):
    tap = models.KegTap.objects.create(site=self._site, name=name,
        meter_name=meter_name, relay_name=relay_name, ml_per_tick=ml_per_tick)
    tap.save()
    return tap

  def CreateAuthToken(self, auth_device, token_value, username=None):
    token = models.AuthenticationToken.objects.create(
        site=self._site, auth_device=auth_device, token_value=token_value)
    if username:
      user = self._GetUserObjFromUsername(username)
      token.user = user
    token.save()
    return token

  def GetAllTaps(self):
    return list(models.KegTap.objects.all())

  def RecordDrink(self, tap_name, ticks, volume_ml=None, username=None,
      pour_time=None, duration=0, shout='', tick_time_series='',
      do_postprocess=True):

    tap = self._GetTapFromName(tap_name)
    if not tap:
      raise BackendError("Tap unknown")

    if volume_ml is None:
      volume_ml = float(ticks) * tap.ml_per_tick

    user = None
    if username:
      user = self._GetUserObjFromUsername(username)
    elif self._site.settings.default_user:
      user = self._site.settings.default_user

    if not pour_time:
      pour_time = timezone.now()

    keg = None
    if tap.current_keg and tap.current_keg.status == 'online':
      keg = tap.current_keg

    if tick_time_series:
      try:
        # Validate the time series by parsing it; canonicalize it by generating
        # it again.  If malformed, just junk it; it's non-essential information.
        tick_time_series = time_series.to_string(time_series.from_string(tick_time_series))
      except ValueError, e:
        self._logger.warning('Time series invalid, ignoring. Error was: %s' % e)
        tick_time_series = ''

    d = models.Drink(ticks=ticks, site=self._site, keg=keg, user=user,
        volume_ml=volume_ml, time=pour_time, duration=duration,
        shout=shout, tick_time_series=tick_time_series)
    models.DrinkingSession.AssignSessionForDrink(d)
    d.save()

    if do_postprocess:
      d.PostProcess()
      event_list = [e for e in models.SystemEvent.objects.filter(drink=d).order_by('id')]
      if settings.HAVE_CELERY:
        tasks.handle_new_events.delay(self._site, event_list)

    return d

  def CancelDrink(self, drink_id, spilled=False):
    try:
      d = self._site.drinks.get(id=drink_id)
    except models.Drink.DoesNotExist:
      return

    keg = d.keg
    user = d.user
    session = d.session

    # Transfer volume to spillage if requested.
    if spilled and d.volume_ml and d.keg:
      d.keg.spilled_ml += d.volume_ml
      d.keg.save()

    d.status = 'deleted'
    d.save()

    # Invalidate all statistics.
    models.SystemStats.objects.filter(site=self._site).delete()
    models.KegStats.objects.filter(site=self._site, keg=d.keg).delete()
    models.UserStats.objects.filter(site=self._site, user=d.user).delete()
    models.SessionStats.objects.filter(site=self._site, session=d.session).delete()

    # Delete any SystemEvents for this drink.
    models.SystemEvent.objects.filter(site=self._site, drink=d).delete()

    # Regenerate new statistics, based on the most recent drink
    # post-cancellation.
    last_qs = self._site.drinks.valid().order_by('-id')
    if last_qs:
      last_drink = last_qs[0]
      last_drink._UpdateSystemStats()

    if keg:
      keg.RecomputeStats()
    if user and user.get_profile():
      user.get_profile().RecomputeStats()
    if session:
      session.Rebuild()
      session.RecomputeStats()

    # TODO(mikey): recompute session.
    return d

  def LogSensorReading(self, sensor_name, temperature, when=None):
    now = timezone.now()
    if not when:
      when = now

    # The maximum resolution of ThermoSensor records is 1 minute.  Round the
    # time down to the nearest minute; if a record already exists for this time,
    # replace it.
    when = when.replace(second=0, microsecond=0)

    # If the temperature is out of bounds, reject it.
    min_val = kb_common.THERMO_SENSOR_RANGE[0]
    max_val = kb_common.THERMO_SENSOR_RANGE[1]
    if temperature < min_val or temperature > max_val:
      raise ValueError('Temperature out of bounds')

    sensor = self._GetSensorFromName(sensor_name)
    defaults = {
        'temp': temperature,
    }
    record, created = models.Thermolog.objects.get_or_create(site=self._site,
        sensor=sensor, time=when, defaults=defaults)
    record.temp = temperature
    record.save()

    # Delete old entries.
    keep_time = now - datetime.timedelta(hours=24)
    old_entries = models.Thermolog.objects.filter(site=self._site, time__lt=keep_time)
    old_entries.delete()

    return record

  def GetAuthToken(self, auth_device, token_value):
    if token_value and auth_device in kb_common.AUTH_MODULE_NAMES_HEX_VALUES:
      token_value = token_value.lower()
    try:
      return models.AuthenticationToken.objects.get(site=self._site,
        auth_device=auth_device, token_value=token_value)
    except models.AuthenticationToken.DoesNotExist:
      raise NoTokenError




from django.db import models
from django.utils.translation import ugettext as _

### CountryField
# Source: http://www.djangosnippets.org/snippets/1281/

COUNTRIES = (
    ('AFG', _('Afghanistan')), 
    ('ALA', _('Aland Islands')),
    ('ALB', _('Albania')),
    ('DZA', _('Algeria')),
    ('ASM', _('American Samoa')),
    ('AND', _('Andorra')),
    ('AGO', _('Angola')),
    ('AIA', _('Anguilla')),
    ('ATG', _('Antigua and Barbuda')),
    ('ARG', _('Argentina')),
    ('ARM', _('Armenia')),
    ('ABW', _('Aruba')),
    ('AUS', _('Australia')),
    ('AUT', _('Austria')),
    ('AZE', _('Azerbaijan')),
    ('BHS', _('Bahamas')),
    ('BHR', _('Bahrain')),
    ('BGD', _('Bangladesh')),
    ('BRB', _('Barbados')),
    ('BLR', _('Belarus')),
    ('BEL', _('Belgium')),
    ('BLZ', _('Belize')),
    ('BEN', _('Benin')),
    ('BMU', _('Bermuda')),
    ('BTN', _('Bhutan')),
    ('BOL', _('Bolivia')),
    ('BIH', _('Bosnia and Herzegovina')),
    ('BWA', _('Botswana')),
    ('BRA', _('Brazil')),
    ('VGB', _('British Virgin Islands')),
    ('BRN', _('Brunei Darussalam')),
    ('BGR', _('Bulgaria')),
    ('BFA', _('Burkina Faso')),
    ('BDI', _('Burundi')),
    ('KHM', _('Cambodia')),
    ('CMR', _('Cameroon')),
    ('CAN', _('Canada')),
    ('CPV', _('Cape Verde')),
    ('CYM', _('Cayman Islands')),
    ('CAF', _('Central African Republic')),
    ('TCD', _('Chad')),
    ('CIL', _('Channel Islands')),
    ('CHL', _('Chile')),
    ('CHN', _('China')),
    ('HKG', _('China - Hong Kong')),
    ('MAC', _('China - Macao')),
    ('COL', _('Colombia')),
    ('COM', _('Comoros')),
    ('COG', _('Congo')),
    ('COK', _('Cook Islands')),
    ('CRI', _('Costa Rica')),
    ('CIV', _('Cote d\'Ivoire')),
    ('HRV', _('Croatia')),
    ('CUB', _('Cuba')),
    ('CYP', _('Cyprus')),
    ('CZE', _('Czech Republic')),
    ('PRK', _('Democratic People\'s Republic of Korea')),
    ('COD', _('Democratic Republic of the Congo')),
    ('DNK', _('Denmark')),
    ('DJI', _('Djibouti')),
    ('DMA', _('Dominica')),
    ('DOM', _('Dominican Republic')),
    ('ECU', _('Ecuador')),
    ('EGY', _('Egypt')),
    ('SLV', _('El Salvador')),
    ('GNQ', _('Equatorial Guinea')),
    ('ERI', _('Eritrea')),
    ('EST', _('Estonia')),
    ('ETH', _('Ethiopia')),
    ('FRO', _('Faeroe Islands')),
    ('FLK', _('Falkland Islands (Malvinas)')),
    ('FJI', _('Fiji')),
    ('FIN', _('Finland')),
    ('FRA', _('France')),
    ('GUF', _('French Guiana')),
    ('PYF', _('French Polynesia')),
    ('GAB', _('Gabon')),
    ('GMB', _('Gambia')),
    ('GEO', _('Georgia')),
    ('DEU', _('Germany')),
    ('GHA', _('Ghana')),
    ('GIB', _('Gibraltar')),
    ('GRC', _('Greece')),
    ('GRL', _('Greenland')),
    ('GRD', _('Grenada')),
    ('GLP', _('Guadeloupe')),
    ('GUM', _('Guam')),
    ('GTM', _('Guatemala')),
    ('GGY', _('Guernsey')),
    ('GIN', _('Guinea')),
    ('GNB', _('Guinea-Bissau')),
    ('GUY', _('Guyana')),
    ('HTI', _('Haiti')),
    ('VAT', _('Holy See (Vatican City)')),
    ('HND', _('Honduras')),
    ('HUN', _('Hungary')),
    ('ISL', _('Iceland')),
    ('IND', _('India')),
    ('IDN', _('Indonesia')),
    ('IRN', _('Iran')),
    ('IRQ', _('Iraq')),
    ('IRL', _('Ireland')),
    ('IMN', _('Isle of Man')),
    ('ISR', _('Israel')),
    ('ITA', _('Italy')),
    ('JAM', _('Jamaica')),
    ('JPN', _('Japan')),
    ('JEY', _('Jersey')),
    ('JOR', _('Jordan')),
    ('KAZ', _('Kazakhstan')),
    ('KEN', _('Kenya')),
    ('KIR', _('Kiribati')),
    ('KWT', _('Kuwait')),
    ('KGZ', _('Kyrgyzstan')),
    ('LAO', _('Lao People\'s Democratic Republic')),
    ('LVA', _('Latvia')),
    ('LBN', _('Lebanon')),
    ('LSO', _('Lesotho')),
    ('LBR', _('Liberia')),
    ('LBY', _('Libyan Arab Jamahiriya')),
    ('LIE', _('Liechtenstein')),
    ('LTU', _('Lithuania')),
    ('LUX', _('Luxembourg')),
    ('MKD', _('Macedonia')),
    ('MDG', _('Madagascar')),
    ('MWI', _('Malawi')),
    ('MYS', _('Malaysia')),
    ('MDV', _('Maldives')),
    ('MLI', _('Mali')),
    ('MLT', _('Malta')),
    ('MHL', _('Marshall Islands')),
    ('MTQ', _('Martinique')),
    ('MRT', _('Mauritania')),
    ('MUS', _('Mauritius')),
    ('MYT', _('Mayotte')),
    ('MEX', _('Mexico')),
    ('FSM', _('Micronesia, Federated States of')),
    ('MCO', _('Monaco')),
    ('MNG', _('Mongolia')),
    ('MNE', _('Montenegro')),
    ('MSR', _('Montserrat')),
    ('MAR', _('Morocco')),
    ('MOZ', _('Mozambique')),
    ('MMR', _('Myanmar')),
    ('NAM', _('Namibia')),
    ('NRU', _('Nauru')),
    ('NPL', _('Nepal')),
    ('NLD', _('Netherlands')),
    ('ANT', _('Netherlands Antilles')),
    ('NCL', _('New Caledonia')),
    ('NZL', _('New Zealand')),
    ('NIC', _('Nicaragua')),
    ('NER', _('Niger')),
    ('NGA', _('Nigeria')),
    ('NIU', _('Niue')),
    ('NFK', _('Norfolk Island')),
    ('MNP', _('Northern Mariana Islands')),
    ('NOR', _('Norway')),
    ('PSE', _('Occupied Palestinian Territory')),
    ('OMN', _('Oman')),
    ('PAK', _('Pakistan')),
    ('PLW', _('Palau')),
    ('PAN', _('Panama')),
    ('PNG', _('Papua New Guinea')),
    ('PRY', _('Paraguay')),
    ('PER', _('Peru')),
    ('PHL', _('Philippines')),
    ('PCN', _('Pitcairn')),
    ('POL', _('Poland')),
    ('PRT', _('Portugal')),
    ('PRI', _('Puerto Rico')),
    ('QAT', _('Qatar')),
    ('KOR', _('Republic of Korea')),
    ('MDA', _('Republic of Moldova')),
    ('REU', _('Reunion')),
    ('ROU', _('Romania')),
    ('RUS', _('Russian Federation')),
    ('RWA', _('Rwanda')),
    ('BLM', _('Saint-Barthelemy')),
    ('SHN', _('Saint Helena')),
    ('KNA', _('Saint Kitts and Nevis')),
    ('LCA', _('Saint Lucia')),
    ('MAF', _('Saint-Martin (French part)')),
    ('SPM', _('Saint Pierre and Miquelon')),
    ('VCT', _('Saint Vincent and the Grenadines')),
    ('WSM', _('Samoa')),
    ('SMR', _('San Marino')),
    ('STP', _('Sao Tome and Principe')),
    ('SAU', _('Saudi Arabia')),
    ('SEN', _('Senegal')),
    ('SRB', _('Serbia')),
    ('SYC', _('Seychelles')),
    ('SLE', _('Sierra Leone')),
    ('SGP', _('Singapore')),
    ('SVK', _('Slovakia')),
    ('SVN', _('Slovenia')),
    ('SLB', _('Solomon Islands')),
    ('SOM', _('Somalia')),
    ('ZAF', _('South Africa')),
    ('ESP', _('Spain')),
    ('LKA', _('Sri Lanka')),
    ('SDN', _('Sudan')),
    ('SUR', _('Suriname')),
    ('SJM', _('Svalbard and Jan Mayen Islands')),
    ('SWZ', _('Swaziland')),
    ('SWE', _('Sweden')),
    ('CHE', _('Switzerland')),
    ('SYR', _('Syrian Arab Republic')),
    ('TJK', _('Tajikistan')),
    ('THA', _('Thailand')),
    ('TLS', _('Timor-Leste')),
    ('TGO', _('Togo')),
    ('TKL', _('Tokelau')),
    ('TON', _('Tonga')),
    ('TTO', _('Trinidad and Tobago')),
    ('TUN', _('Tunisia')),
    ('TUR', _('Turkey')),
    ('TKM', _('Turkmenistan')),
    ('TCA', _('Turks and Caicos Islands')),
    ('TUV', _('Tuvalu')),
    ('UGA', _('Uganda')),
    ('UKR', _('Ukraine')),
    ('ARE', _('United Arab Emirates')),
    ('GBR', _('United Kingdom')),
    ('TZA', _('United Republic of Tanzania')),
    ('USA', _('United States of America')),
    ('VIR', _('United States Virgin Islands')),
    ('URY', _('Uruguay')),
    ('UZB', _('Uzbekistan')),
    ('VUT', _('Vanuatu')),
    ('VEN', _('Venezuela (Bolivarian Republic of)')),
    ('VNM', _('Viet Nam')),
    ('WLF', _('Wallis and Futuna Islands')),
    ('ESH', _('Western Sahara')),
    ('YEM', _('Yemen')),
    ('ZMB', _('Zambia')),
    ('ZWE', _('Zimbabwe')),
)

class CountryField(models.CharField):
    def __init__(self, *args, **kwargs):
        kwargs.setdefault('max_length', 3)
        kwargs.setdefault('choices', COUNTRIES)

        super(CountryField, self).__init__(*args, **kwargs)

    def get_internal_type(self):
        return "CharField"

try:
    from south.modelsinspector import add_introspection_rules
except ImportError:
    pass
else:
    add_introspection_rules([
        (
            [CountryField],
            [],
            {},
        ),
    ], ["^pykeg\.core\.fields\.CountryField"])




#!/usr/bin/env python

import unittest
from pykeg.core import time_series

class TimeSeriesTestCase(unittest.TestCase):
  def testTimeSeries(self):
    s = "  0:12 45:6789  "
    expected = [ (0, 12), (45, 6789) ]
    self.assertEqual(expected, time_series.from_string(s))

    self.assertEqual(s.strip(), time_series.to_string(expected))

if __name__ == '__main__':
  unittest.main()



# Copyright 2013 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

from django import forms
from crispy_forms.helper import FormHelper
from crispy_forms.layout import Layout, Div, Submit, HTML, Button, Row, Field
from crispy_forms.bootstrap import AppendedText, PrependedText, FormActions

from pykeg.core import models

class CreateOrImportForm(forms.Form):
  CHOICES = (
      ('create', 'Create New System'),
      ('import', 'Import From Backup'),
  )
  mode = forms.ChoiceField(widget=forms.RadioSelect, choices=CHOICES)
  import_file  = forms.FileField(required=False)

  helper = FormHelper()
  helper.form_class = 'setup-form span8 offset2'
  helper.layout = Layout(
      Field('mode', css_class='input-xlarge'),
      Field('import_file', css_class='input-xlarge'),
      FormActions(
          Submit('save_changes', 'Continue', css_class="btn-primary"),
      )
  )


class MiniSiteSettingsForm(forms.ModelForm):
  class Meta:
    model = models.SiteSettings
    fields = (
        'title',
        'privacy',
        'volume_display_units',
        'temperature_display_units',
        'registration_allowed',
        'registration_confirmation',
    )
    widgets = {
        'privacy': forms.RadioSelect(),
        'volume_display_units': forms.RadioSelect(),
        'temperature_display_units': forms.RadioSelect(),
    }

  helper = FormHelper()
  helper.form_class = 'setup-form span8 offset2'
  helper.layout = Layout(
      Field('title', css_class='span12'),
      Field('privacy', css_class='span12'),
      Field('volume_display_units', css_class='span12'),
      Field('temperature_display_units', css_class='span12'),
      Field('registration_allowed'),
      Field('registration_confirmation'),
      FormActions(
          Submit('save_changes', 'Continue', css_class="btn-primary"),
      )
  )

class AdminUserForm(forms.Form):
  username = forms.RegexField(
      label='Username',
      max_length=30,
      regex=r'^[\w-]+$',
      help_text='Your username: 30 characters or fewer. Alphanumeric characters only (letters, digits, hyphens and underscores).',
      error_message='Must contain only letters, numbers, hyphens and underscores.')

  password = forms.CharField(widget=forms.PasswordInput())
  confirm_password = forms.CharField(widget=forms.PasswordInput())
  email = forms.EmailField(
      help_text='In case you lose your password.')

  helper = FormHelper()
  helper.form_class = 'setup-form span8 offset2'
  helper.layout = Layout(
      Field('username', css_class='span12'),
      Field('email', css_class='span12'),
      Field('password', css_class='span12'),
      Field('confirm_password', css_class='span12'),
      FormActions(
          Submit('save_changes', 'Continue', css_class="btn-primary"),
      )
  )

  def clean_confirm_password(self):
    orig = self.cleaned_data.get('password')
    confirm = self.cleaned_data.get('confirm_password')
    if orig and confirm and orig != confirm:
      raise forms.ValidationError('Passwords must match!')
    return confirm

  def clean_username(self):
    username = self.cleaned_data.get('username')
    if username:
      try:
        models.User.objects.get(username=username)
        raise forms.ValidationError('Sorry, this username is taken already?!')
      except models.User.DoesNotExist:
        pass  # expected
    return username

  def save(self):
    u = models.User()
    u.username = self.cleaned_data.get('username')
    u.email = self.cleaned_data.get('email')
    u.is_staff = True
    u.is_superuser = True
    u.save()
    u.set_password(self.cleaned_data.get('password'))
    u.save()
    return u




from django import forms
from pykeg.contrib.facebook import models as facebook_models

class FacebookSettingsForm(forms.ModelForm):
  class Meta:
    model = facebook_models.FacebookSettings
    exclude = ('profile',)

class WallPostForm(forms.Form):
  message = forms.CharField()



# Copyright 2012 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Utilities for processing API views."""

from django.conf import settings
from django.http import Http404
from django.http import HttpResponse
from django.db.models.query import QuerySet
from pykeg.proto import protolib
from kegbot.api import protoutil
from google.protobuf.message import Message

from kegbot.api import kbapi
from kegbot.util import kbjson
from pykeg.core import models
from pykeg.core import backend

from . import validate_jsonp

import logging
import sys
import traceback
import types

LOGGER = logging.getLogger(__name__)

def is_api_request(request):
  return request.path.startswith('/api')

def check_api_key(request):
  """Check a request for an API key."""
  keystr = request.META.get('HTTP_X_KEGBOT_API_KEY')
  if not keystr:
    keystr = request.REQUEST.get('api_key')
  if not keystr:
    raise kbapi.NoAuthTokenError('The parameter "api_key" is required')

  try:
    api_key = models.ApiKey.objects.get(key=keystr)
  except models.ApiKey.DoesNotExist:
    raise kbapi.BadApiKeyError('API key does not exist')

  if not api_key.is_active():
    raise kbapi.BadApiKeyError('Key and/or user is inactive')

  if not api_key.user or not api_key.user.is_active:
    raise kbapi.PermissionDeniedError('User is disabled or does not exist.')

  # TODO: remove me.
  if not api_key.user.is_staff and not api_key.user.is_superuser:
    raise kbapi.PermissionDeniedError('User is not staff/superuser')

def to_json_error(e, exc_info):
  """Converts an exception to an API error response."""
  # Wrap some common exception types into kbapi types
  if isinstance(e, Http404):
    e = kbapi.NotFoundError(e.message)
  elif isinstance(e, ValueError):
    e = kbapi.BadRequestError(str(e))
  elif isinstance(e, backend.NoTokenError):
    e = kbapi.NotFoundError(e.message)

  # Now determine the response based on the exception type.
  if isinstance(e, kbapi.Error):
    code = e.__class__.__name__
    http_code = e.HTTP_CODE
    message = e.Message()
  else:
    code = 'ServerError'
    http_code = 500
    message = 'An internal error occurred: %s' % str(e)
  result = {
    'error' : {
      'code' : code,
      'message' : message
    }
  }
  if settings.DEBUG:
    result['error']['traceback'] = "".join(traceback.format_exception(*exc_info))
  return result, http_code

def build_response(result_data, response_code=200, callback=None):
  """Builds an HTTP response for JSON data."""
  indent = 2
  json_str = kbjson.dumps(result_data, indent=indent)
  if callback and validate_jsonp.is_valid_jsonp_callback_value(callback):
    json_str = '%s(%s);' % (callback, json_str)
  return HttpResponse(json_str, mimetype='application/json', status=response_code)


def prepare_data(data, inner=False):
  if isinstance(data, QuerySet) or type(data) == types.ListType:
    result = [prepare_data(d, True) for d in data]
    container = 'objects'
  elif isinstance(data, dict):
    result = data
    container = 'object'
  else:
    result = to_dict(data)
    container = 'object'

  if inner:
    return result
  else:
    return {
      container: result
    }

def to_dict(data):
  if not isinstance(data, Message):
    data = protolib.ToProto(data, full=True)
  return protoutil.ProtoMessageToDict(data)

def wrap_exception(request, exception):
  """Returns a HttpResponse with the exception in JSON form."""
  exc_info = sys.exc_info()

  LOGGER.error('%s: %s' % (exception.__class__.__name__, exception),
      exc_info=exc_info,
      extra={
        'status_code': 500,
        'request': request,
      }
  )

  if settings.DEBUG and settings.HAVE_RAVEN:
    from raven.contrib.django.models import client
    client.captureException()

  # Don't wrap the exception during debugging.
  if settings.DEBUG and 'deb' in request.GET:
    return None

  result_data, http_code = to_json_error(exception, exc_info)
  result_data['meta'] = {
    'result': 'error'
  }
  return build_response(result_data, response_code=http_code)





#!/usr/bin/env python
#
# Copyright 2010 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

import cStringIO
import datetime

from django.contrib import messages
from django.contrib.admin.views.decorators import staff_member_required
from django.contrib.auth.decorators import login_required
from django.core.urlresolvers import reverse
from django.db.models import Q
from django.http import HttpResponse
from django.shortcuts import get_object_or_404
from django.shortcuts import redirect
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.forms import widgets
from django.views.decorators.http import require_POST
from django.utils import timezone
from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger

from kegbot.util import kbjson

from pykeg.core import backup
from pykeg.core import logger
from pykeg.core import models
from pykeg.connections.foursquare import forms as foursquare_forms
from pykeg.connections.foursquare import models as foursquare_models
from pykeg.connections.twitter import models as twitter_models
from pykeg.connections.twitter import forms as twitter_forms
from pykeg.connections.twitter import util as twitter_util

from pykeg.web.kegadmin import forms

@staff_member_required
def dashboard(request):
  context = RequestContext(request)
  recent_time = timezone.now() - datetime.timedelta(days=30)
  active_users = models.User.objects.filter(is_active=True)
  new_users = models.User.objects.filter(date_joined__gte=recent_time)
  context['num_users'] = len(active_users)
  context['num_new_users'] = len(new_users)
  return render_to_response('kegadmin/dashboard.html', context)

@staff_member_required
def general_settings(request):
  context = RequestContext(request)
  form = forms.SiteSettingsForm(instance=request.kbsite.settings)
  if request.method == 'POST':
    form = forms.SiteSettingsForm(request.POST, instance=request.kbsite.settings)
    if form.is_valid():
      form.save()
      messages.success(request, 'Site settings were successfully updated.')
  context['settings_form'] = form
  return render_to_response('kegadmin/index.html', context)

@staff_member_required
def tap_list(request):
  context = RequestContext(request)
  context['taps'] = request.kbsite.taps.all()
  return render_to_response('kegadmin/tap_list.html', context)

@staff_member_required
def add_tap(request):
  context = RequestContext(request)
  form = forms.TapForm(site=request.kbsite)
  if request.method == 'POST':
    form = forms.TapForm(request.POST, site=request.kbsite)
    if form.is_valid():
      new_tap = form.save(commit=False)
      new_tap.site = request.kbsite
      new_tap.save()
      messages.success(request, 'Tap created.')
      return redirect('kegadmin-taps')
  context['form'] = form
  return render_to_response('kegadmin/add_tap.html', context)

@staff_member_required
def tap_detail(request, tap_id):
  tap = get_object_or_404(models.KegTap, site=request.kbsite, id=tap_id)

  activate_keg_form = forms.ChangeKegForm()
  tap_settings_form = forms.TapForm(instance=tap, site=request.kbsite)

  if request.method == 'POST':
    if 'submit_change_keg_form' in request.POST:
      activate_keg_form = forms.ChangeKegForm(request.POST)
      if activate_keg_form.is_valid():
        activate_keg_form.save(tap)
        messages.success(request, 'The new keg was activated. Bottoms up!')
        return redirect('kegadmin-taps')
    
    elif 'submit_tap_form' in request.POST:
      tap_settings_form = forms.TapForm(request.POST, instance=tap, site=request.kbsite)
      if tap_settings_form.is_valid():
        tap_settings_form.save()
        messages.success(request, 'Tap settings saved.')
        tap_settings_form = forms.TapForm(instance=tap, site=request.kbsite)

    elif 'submit_delete_tap_form' in request.POST:
      delete_form = forms.DeleteTapForm(request.POST)
      if delete_form.is_valid():
        if tap.current_keg:
          tap.current_keg.status = 'offline'
          tap.current_keg.save()
        tap.delete()
        messages.success(request, 'Tap deleted.')
        return redirect('kegadmin-taps')

    elif 'submit_end_keg_form' in request.POST:
      end_keg_form = forms.EndKegForm(request.POST)
      if end_keg_form.is_valid():
        tap.current_keg = None
        tap.save()
        keg = end_keg_form.cleaned_data['keg']
        keg.status = 'offline'
        keg.save()
        messages.success(request, 'Keg %s was ended.' % keg.id)

    else:
      messages.warning(request, 'No form data was found. Bug?')

  end_keg_form = forms.EndKegForm(initial={'keg':tap.current_keg})

  context = RequestContext(request)
  context['tap'] = tap
  context['current_keg'] = tap.current_keg
  context['activate_keg_form'] = activate_keg_form
  context['end_keg_form'] = end_keg_form
  context['tap_settings_form'] = tap_settings_form
  context['delete_tap_form'] = forms.DeleteTapForm()
  return render_to_response('kegadmin/tap_detail.html', context)

@staff_member_required
def user_list(request):
  context = RequestContext(request)

  if request.method == 'POST':
    form = forms.FindUserForm(request.POST)
    if form.is_valid():
      username = form.cleaned_data.get('username')
      try:
        user = models.User.objects.get(username=username)
        return redirect('kegadmin-edit-user', user.id)
      except models.User.DoesNotExist:
        messages.error(request, 'User "%s" does not exist.' % username)

  users = models.User.objects.all().order_by('-id')
  paginator = Paginator(users, 25)

  page = request.GET.get('page')
  try:
    users = paginator.page(page)
  except PageNotAnInteger:
    users = paginator.page(1)
  except EmptyPage:
    users = paginator.page(paginator.num_pages)

  context['users'] = users
  return render_to_response('kegadmin/user_list.html', context)

@staff_member_required
def add_user(request):
  context = RequestContext(request)
  form = forms.UserForm()
  if request.method == 'POST':
    form = forms.UserForm(request.POST)
    if form.is_valid():
      instance = form.save(commit=False)
      instance.set_password(form.cleaned_data.get('password'))
      instance.save()
      messages.success(request, 'User "%s" created.' % instance.username)
      return redirect('kegadmin-users')
  context['form'] = form
  return render_to_response('kegadmin/add_user.html', context)

@staff_member_required
def user_detail(request, user_id):
  user = get_object_or_404(models.User, id=user_id)

  if request.method == 'POST':
    if 'submit_enable' in request.POST:
      if user.is_active:
        messages.error(request, 'User is already enabled.')
      else:
        user.is_active = True
        user.save()
        messages.success(request, 'User %s was enabled.' % user.username)

    elif 'submit_disable' in request.POST:
      if not user.is_active:
        messages.error(request, 'User is already disabled.')
      else:
        user.is_active = False
        user.save()
        messages.success(request, 'User %s was disabled.' % user.username)

  context = RequestContext(request)
  context['user'] = user
  context['profile'] = user.get_profile()
  context['stats'] = context['profile'].GetStats()

  context['tokens'] = user.tokens.all().order_by('created_time')

  return render_to_response('kegadmin/user_detail.html', context)

@staff_member_required
def token_list(request):
  context = RequestContext(request)
  tokens = models.AuthenticationToken.objects.all().order_by('-created_time')
  paginator = Paginator(tokens, 25)

  page = request.GET.get('page')
  try:
    tokens = paginator.page(page)
  except PageNotAnInteger:
    tokens = paginator.page(1)
  except EmptyPage:
    tokens = paginator.page(paginator.num_pages)

  context['tokens'] = tokens
  return render_to_response('kegadmin/token_list.html', context)


@staff_member_required
def token_detail(request, token_id):
  token = get_object_or_404(models.AuthenticationToken, id=token_id)

  username = ''
  if token.user:
    username = token.user.username

  form = forms.TokenForm(instance=token, initial={'username': username})
  if request.method == 'POST':
    form = forms.TokenForm(request.POST, instance=token)
    if form.is_valid():
      instance = form.save(commit=False)
      instance.user = form.cleaned_data['user']
      instance.save()
      messages.success(request, 'Token updated.')

  context = RequestContext(request)
  context['token'] = token
  context['form'] = form
  return render_to_response('kegadmin/token_detail.html', context)

@staff_member_required
def add_token(request):
  context = RequestContext(request)
  form = forms.AddTokenForm()
  if request.method == 'POST':
    form = forms.AddTokenForm(request.POST)
    if form.is_valid():
      instance = form.save(commit=False)
      instance.user = form.cleaned_data['user']
      instance.site = request.kbsite
      instance.save()
      messages.success(request, 'Token created.')
      return redirect('kegadmin-tokens')
  context['form'] = form
  return render_to_response('kegadmin/add_token.html', context)


@staff_member_required
def connections(request):
  context = RequestContext(request)
  kbsite = request.kbsite

  try:
    twitter_profile = request.kbsite.twitter_profile
  except twitter_models.SiteTwitterProfile.DoesNotExist:
    twitter_profile = None

  tweet_form = forms.TweetForm()
  if request.method == 'POST' and 'tweet-form-submit' in request.POST:
    tweet_form = forms.TweetForm(request.POST)
    if tweet_form.is_valid():
      if kbsite.twitter_profile.is_enabled():
        tweet = tweet_form.cleaned_data['tweet']
        twitter_util.update_site_status(kbsite, tweet)
        tweet_form = forms.TweetForm()
        messages.success(request, 'Tweetz0red!')
      else:
        messages.error(request, 'Twitter profile not enabled.')

  if twitter_profile:
    instance, _ = twitter_models.SiteTwitterSettings.objects.get_or_create(profile=twitter_profile)
    twitter_settings_form = twitter_forms.SiteTwitterSettingsForm(instance=instance)
    if request.method == 'POST' and 'twitter-settings-submit' in request.POST:
      twitter_settings_form = twitter_forms.SiteTwitterSettingsForm(request.POST, instance=instance)
      if twitter_settings_form.is_valid():
        twitter_settings_form.save()
      else:
        messages.error(request, 'Twitter settings form had errors.')
    context['twitter_settings_form'] = twitter_settings_form

  instance, _ = foursquare_models.SiteFoursquareSettings.objects.get_or_create(site=kbsite)
  foursquare_form = foursquare_forms.SiteFoursquareSettingsForm(instance=instance)
  if request.method == 'POST' and 'foursquare-settings-submit' in request.POST:
    foursquare_form = foursquare_forms.SiteFoursquareSettingsForm(request.POST, instance=instance)
    if foursquare_form.is_valid():
      foursquare_form.save()
      messages.success(request, 'Foursquare settings updated!')

  context['twitter_profile'] = twitter_profile
  context['tweet_form'] = tweet_form
  context['foursquare_settings_form'] = foursquare_form
  return render_to_response('kegadmin/connections.html', context)

@staff_member_required
def autocomplete_beer_type(request):
  context = RequestContext(request)
  search = request.GET.get('q')
  if search:
    types = models.BeerType.objects.filter(Q(name__icontains=search) | Q(brewer__name__icontains=search))
  else:
    types = models.BeerType.objects.all()
  types = types[:10]  # autocomplete widget limited to 10
  values = []
  for beer in types:
    values.append({
      'name': beer.name,
      'id': beer.id,
      'brewer_name': beer.brewer.name,
      'brewer_id': beer.brewer.id,
      'style_name': beer.style.name,
      'style_id': beer.style.id,
  })
  return HttpResponse(kbjson.dumps(values, indent=None),
    mimetype='application/json', status=200)

@staff_member_required
def autocomplete_user(request):
  context = RequestContext(request)
  search = request.GET.get('q')
  if search:
    users = models.User.objects.filter(Q(username__icontains=search) | Q(email__icontains=search))
  else:
    users = models.User.objects.all()
  users = users[:10]  # autocomplete widget limited to 10
  values = []
  for user in users:
    values.append({
      'username': user.username,
      'id': user.id,
      'email': user.email,
      'is_active': user.is_active,
  })
  return HttpResponse(kbjson.dumps(values, indent=None),
    mimetype='application/json', status=200)

@staff_member_required
def autocomplete_token(request):
  context = RequestContext(request)
  search = request.GET.get('q')
  if search:
    tokens = models.AuthenticationToken.objects.filter(
        Q(token_value__icontains=search) | Q(nice_name__icontains=search))
  else:
    tokens = models.AuthenticationToken.objects.all()
  tokens = tokens[:10]  # autocomplete widget limited to 10
  values = []
  for token in tokens:
    values.append({
      'username': token.user.username,
      'id': token.id,
      'auth_device': token.auth_device,
      'token_value': token.token_value,
      'enabled': token.enabled,
  })
  return HttpResponse(kbjson.dumps(values, indent=None),
    mimetype='application/json', status=200)

@staff_member_required
def backup_restore(request):
  context = RequestContext(request)
  return render_to_response('kegadmin/backup-restore.html', context)

@staff_member_required
def generate_backup(request):
  context = RequestContext(request)

  fmt = request.GET.get('fmt', 'minjson')
  if fmt == 'json':
    indent = 2
  else:
    indent = None
    fmt = 'minjson'

  kbsite = request.kbsite
  datestr = timezone.now().strftime('%Y%m%d-%H%M%S')
  filename = 'kegbot-%s.%s.%s.txt' % (kbsite.name, datestr, fmt)

  output_fp = cStringIO.StringIO()
  backup.dump(output_fp, kbsite, indent=indent)

  response = HttpResponse(output_fp.getvalue(),
      mimetype="application/octet-stream")
  response['Content-Disposition'] = 'attachment; filename=%s' % filename
  output_fp.close()
  return response

@staff_member_required
def logs(request):
  context = RequestContext(request)
  context['errors'] = logger.get_cached_logs()
  return render_to_response('kegadmin/logs.html', context)




from django.conf.urls import patterns
from django.conf.urls import url

from pykeg.core import features
from pykeg.web.account.views import password_change
from pykeg.web.account.views import password_change_done
from pykeg.web.account.views import remove_untappd


urlpatterns = patterns('pykeg.web.account.views',
    url(r'^$', 'account_main', name='kb-account-main'),
    url(r'^connections/$', 'connections', name='account-connections'),
    url(r'^password/done/$', password_change_done, name='password_change_done'),
    url(r'^password/$', password_change, name='password_change'),
    url(r'^mugshot/$', 'edit_mugshot', name='account-mugshot'),
    url(r'^regenerate-api-key/$', 'regenerate_api_key', name='regen-api-key'),
    url(r'^update-foursquare-settings/$', 'update_foursquare_settings', name='update-foursquare-settings'),
    url(r'^remove-foursquare/$', 'remove_foursquare', name='remove-foursquare'),
    url(r'^update-twitter-settings/$', 'update_twitter_settings', name='update-twitter-settings'),
    url(r'^remove-twitter/$', 'remove_twitter', name='remove-twitter'),
)

if features.use_facebook():
  urlpatterns += patterns('',
      url(r'fb-settings/$', 'pykeg.web.contrib.facebook.views.account_settings', name='fb-account-settings'),
  )

if features.use_untappd():
  urlpatterns += patterns('',
      url(r'^remove-untappd/$', remove_untappd, name='remove-untappd'),
  )



# Copyright 2012 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

from django.conf import settings
from django.http import HttpResponse

from . import util

import logging
import sys

LOGGER = logging.getLogger(__name__)

def wrap_exception(request, exception):
  """Returns a HttpResponse with the exception in JSON form."""
  exc_info = sys.exc_info()

  LOGGER.error('%s: %s' % (exception.__class__.__name__, exception),
      exc_info=exc_info,
      extra={
        'status_code': 500,
        'request': request,
      }
  )

  if settings.DEBUG and settings.HAVE_RAVEN:
    from raven.contrib.django.models import client
    client.captureException()

  # Don't wrap the exception during debugging.
  if settings.DEBUG and 'deb' in request.GET:
    return None

  result_data, http_code = util.to_json_error(exception, exc_info)
  result_data['meta'] = {
    'result': 'error'
  }
  return util.build_response(result_data, response_code=http_code)


class ApiRequestMiddleware:
  def process_request(self, request):
    request.is_kb_api_request = util.is_api_request(request)
    if not request.is_kb_api_request:
      # Not an API request. Skip me.
      return None
    elif request.kbsite.settings.privacy == 'public':
      # API request, but public site privacy.  Views will check access as needed.
      return None
    elif request.path in ('/api/login/', '/api/get-api-key/'):
      # API request to whitelisted path.
      return None
    else:
      # API request to non-whitelisted path, in non-public site privacy mode.
      # Demand API key.
      if request.user.is_authenticated:
        return None
      elif request.kbsite.settings.privacy == 'staff' and request.user.is_staff:
        return None
      else:
        try:
          util.check_api_key(request)
        except Exception, e:
          return wrap_exception(request, e)


class ApiResponseMiddleware:
  def process_exception(self, request, exception):
    """Wraps exceptions for API requests."""
    if util.is_api_request(request):
      return wrap_exception(request, exception)
    return None

  def process_response(self, request, response):
    if not util.is_api_request(request):
      return response

    if not isinstance(response, HttpResponse):
      data = util.prepare_data(response)
      data['meta'] = {
        'result': 'ok'
      }
      callback = request.GET.get('callback')
      response = util.build_response(data, 200, callback=callback)
    response['Cache-Control'] = 'max-age=0'
    return response
